from ctypes import *
import os
import math
import random
import numpy as np
import sys
import itertools
from subprocess import call, Popen
from multiprocessing import Process, Queue
import cv2 
import copy, shutil
import codecs
from os import listdir
from os.path import isfile, join
from sklearn import metrics as sklearn_metrics


def sample(probs):
    s = sum(probs)
    probs = [a / s for a in probs]
    r = random.uniform(0, 1)
    for i in range(len(probs)):
        r = r - probs[i]
        if r <= 0:
            return i
    return len(probs) - 1


def c_array(ctype, values):
    arr = (ctype * len(values))()
    arr[:] = values
    return arr


class BOX(Structure):
    _fields_ = [("x", c_float),
                ("y", c_float),
                ("w", c_float),
                ("h", c_float)]


class DETECTION(Structure):
    _fields_ = [("bbox", BOX),
                ("classes", c_int),
                ("prob", POINTER(c_float)),
                ("mask", POINTER(c_float)),
                ("objectness", c_float),
                ("sort_class", c_int)]


class IMAGE(Structure):
    _fields_ = [("w", c_int),
                ("h", c_int),
                ("c", c_int),
                ("data", POINTER(c_float))]


class METADATA(Structure):
    _fields_ = [("classes", c_int),
                ("names", POINTER(c_char_p))]


lib = CDLL("/media/HDD3TB/pritomrit/YOLO_weights/libdarknet.so",
           RTLD_GLOBAL)
# lib = CDLL("libdarknet.so", RTLD_GLOBAL)
lib.network_width.argtypes = [c_void_p]
lib.network_width.restype = c_int
lib.network_height.argtypes = [c_void_p]
lib.network_height.restype = c_int

predict = lib.network_predict
predict.argtypes = [c_void_p, POINTER(c_float)]
predict.restype = POINTER(c_float)

set_gpu = lib.cuda_set_device
set_gpu.argtypes = [c_int]

make_image = lib.make_image
make_image.argtypes = [c_int, c_int, c_int]
make_image.restype = IMAGE

get_network_boxes = lib.get_network_boxes
get_network_boxes.argtypes = [c_void_p, c_int, c_int, c_float, c_float, POINTER(c_int), c_int, POINTER(c_int)]
get_network_boxes.restype = POINTER(DETECTION)

make_network_boxes = lib.make_network_boxes
make_network_boxes.argtypes = [c_void_p]
make_network_boxes.restype = POINTER(DETECTION)

free_detections = lib.free_detections
free_detections.argtypes = [POINTER(DETECTION), c_int]

free_ptrs = lib.free_ptrs
free_ptrs.argtypes = [POINTER(c_void_p), c_int]

network_predict = lib.network_predict
network_predict.argtypes = [c_void_p, POINTER(c_float)]

reset_rnn = lib.reset_rnn
reset_rnn.argtypes = [c_void_p]

load_net = lib.load_network
load_net.argtypes = [c_char_p, c_char_p, c_int]
load_net.restype = c_void_p

do_nms_obj = lib.do_nms_obj
do_nms_obj.argtypes = [POINTER(DETECTION), c_int, c_int, c_float]

do_nms_sort = lib.do_nms_sort
do_nms_sort.argtypes = [POINTER(DETECTION), c_int, c_int, c_float]

free_image = lib.free_image
free_network = lib.free_network
# free_network.argtypes = POINTER(NETWORK)
free_image.argtypes = [IMAGE]

letterbox_image = lib.letterbox_image
letterbox_image.argtypes = [IMAGE, c_int, c_int]
letterbox_image.restype = IMAGE

load_meta = lib.get_metadata
lib.get_metadata.argtypes = [c_char_p]
lib.get_metadata.restype = METADATA

load_image = lib.load_image_color
load_image.argtypes = [c_char_p, c_int, c_int]
load_image.restype = IMAGE

rgbgr_image = lib.rgbgr_image
rgbgr_image.argtypes = [IMAGE]

predict_image = lib.network_predict_image
predict_image.argtypes = [c_void_p, IMAGE]
predict_image.restype = POINTER(c_float)


def classify(net, meta, im):
    out = predict_image(net, im)
    res = []
    for i in range(meta.classes):
        res.append((meta.names[i], out[i]))
    res = sorted(res, key=lambda x: -x[1])
    return res


def detect(net, meta, image, thresh=.5, hier_thresh=.5, nms=.45):
    im = load_image(image, 0, 0)
    num = c_int(0)
    pnum = pointer(num)
    predict_image(net, im)
    dets = get_network_boxes(net, im.w, im.h, thresh, hier_thresh, None, 0, pnum)
    num = pnum[0]
    if (nms): do_nms_obj(dets, num, meta.classes, nms);

    res = []
    for j in range(num):
        for i in range(meta.classes):
            if dets[j].prob[i] > 0:
                b = dets[j].bbox
                res.append((meta.names[i], dets[j].prob[i], (b.x, b.y, b.w, b.h)))
    res = sorted(res, key=lambda x: -x[1])
    free_image(im)
    free_detections(dets, num)
    return res


def is_bb_overlap(boxA, boxB):
    isOverlap = True

    minXA, maxXA = boxA[0] - boxA[2] * .5, boxA[0] + boxA[2] * .5
    minYA, maxYA = boxA[1] - boxA[3] * .5, boxA[1] + boxA[3] * .5

    minXB, maxXB = boxB[0] - boxB[2] * .5, boxB[0] + boxB[2] * .5
    minYB, maxYB = boxB[1] - boxB[3] * .5, boxB[1] + boxB[3] * .5

    if (minXB > maxXA or maxXB < minXA or minYB > maxYA or maxYB < minYA):
        isOverlap = False

    return isOverlap


def intersection_over_minbbox(boxA, boxB):
    # determine the (x, y)-coordinates of the intersection rectangle
    xA = max(boxA[0] - boxA[2] * .5, boxB[0] - boxB[2] * .5)
    yA = max(boxA[1] - boxA[3] * .5, boxB[1] - boxB[3] * .5)
    xB = min(boxA[0] + boxA[2] * .5, boxB[0] + boxB[2] * .5)
    yB = min(boxA[1] + boxA[3] * .5, boxB[1] + boxB[3] * .5)

    # compute the area of intersection rectangle
    interArea = abs(xB - xA) * abs(yB - yA)

    # compute the area of both the prediction and ground-truth
    # rectangles
    boxAArea = boxA[2] * boxA[3]
    boxBArea = boxB[2] * boxB[3]

    boxArea = np.minimum(boxAArea, boxBArea)
    # compute the intersection over union by taking the intersection
    # area and dividing it by the sum of prediction + ground-truth
    # areas - the interesection area
    intersection = interArea / float(boxArea)

    # return the intersection over union value
    return intersection


def custom_detect(net, meta, imgs_list, thresh=.5, hier_thresh=.5, nms=.45, store_root=""):
    with open(imgs_list, "r") as f:
        for line in f:
            image = line.strip()
            #print
            image
            img_name = image.split("/")[-1].split(".")[0]
            im = load_image(image, 0, 0)
            num = c_int(0)
            pnum = pointer(num)
            predict_image(net, im)
            dets = get_network_boxes(net, im.w, im.h, thresh, hier_thresh, None, 0, pnum)
            num = pnum[0]
            if (nms): do_nms_obj(dets, num, meta.classes, nms);

            w, h = im.w, im.h
            res = []
            for j in range(num):
                for i in range(meta.classes):
                    if dets[j].prob[i] > 0:
                        b = dets[j].bbox
                        lbl = dic_classes[yolo_classes_list[int(i)]]
                        str_dets = " ".join([str(lbl), str(b.x / w), str(b.y / h), str(b.w / w), str(b.h / h),
                                             str(dets[j].prob[i])]) + "\n"
                        res.append(str_dets)
            #print
            np.shape(res)
            with open(store_root + "/" + img_name + ".txt", "w") as f2:
                f2.writelines(res)
            # res = sorted(res, key=lambda x: -x[1])
            free_image(im)
            free_detections(dets, num)
    # return res


def remove_duplicate_detections(imgs_list, store_root, nms_thresh):

    with open(imgs_list, "r") as f:
        for line in f:
            image = line.strip()
            #print
            image
            img_name = image.split("/")[-1].split(".")[0]
            img_preds = store_root + "/" + img_name  + ".txt"

            p_f = open(img_preds, "r")
            p_list = [line.strip() for line in p_f]
            p_f.close()

            lbl_pred_list = []
            bbs_pred = []
            prob_pred = []

            for pred in p_list:
                lbl, x, y, bw, bh, prob = pred.split(" ")
                lbl_pred_list.append(lbl)
                boxA = [float(x), float(y), float(bw), float(bh)]
                bbs_pred.append(boxA)
                prob_pred.append(float(prob))

            aux_lbl_pred_list = lbl_pred_list
            aux_bbs_pred = bbs_pred
            aux_prob_pred = prob_pred

            sort1 = np.argsort(aux_prob_pred)[::-1]
            aux_prob_pred = list(np.array(aux_prob_pred)[sort1])
            aux_lbl_pred_list = list(np.array(aux_lbl_pred_list)[sort1])
            aux_bbs_pred = list(np.array(aux_bbs_pred)[sort1])
            selected_bbox = []
            selected_label = []
            init_index = 0
            for k in range(len(aux_lbl_pred_list) - 1):
                if aux_lbl_pred_list[k] == None:
                    continue
                selected_bbox.append(aux_bbs_pred[k])
                selected_label.append(int(aux_lbl_pred_list[k]))
                init_index = k + 1
                break
            for i in range(init_index, len(aux_lbl_pred_list)):
                if aux_lbl_pred_list[i] == None:
                    continue
                boxB = aux_bbs_pred[i]
                lblB = int(aux_lbl_pred_list[i])
                flag = False
                for j in range(len(selected_bbox)):
                    boxA = selected_bbox[j]
                    lblA = int(selected_label[j])
                    if is_bb_overlap(boxA, boxB) and \
                            ((intersection_over_minbbox(boxA, boxB) > nms_thresh and lblA != lblB)):
                        aux_lbl_pred_list[i] = None
                        aux_bbs_pred[i] = None
                        aux_prob_pred[i] = None
                        flag = True
                        break
                if not flag:
                    selected_bbox.append(boxB)
                    selected_label.append(lblB)

            lbl_pred_list = []
            bbs_pred = []
            prob_pred = []
            for j in range(len(aux_lbl_pred_list)):
                if aux_lbl_pred_list[j] != None:
                    lbl_pred_list.append(aux_lbl_pred_list[j])
                    bbs_pred.append(aux_bbs_pred[j])
                    prob_pred.append(aux_prob_pred[j])

            res = []
            for k in range(len(prob_pred)):
                lbl = lbl_pred_list[k]
                bbs = bbs_pred[k]
                prob = prob_pred[k]
                str_dets = " ".join([str(lbl), str(bbs[0]), str(bbs[1]), str(bbs[2]), str(bbs[3]), str(prob)]) + "\n"
                res.append(str_dets)
            with open(store_root + "/" + img_name + ".txt", "w") as f2:
                f2.writelines(res)

            #print "------------------>", np.shape(prob_pred)


def convert_format():
    pass


def detect_fold(cfg_path, weights_path, data_path, thresh, nms, store_root):
    net = load_net(cfg_path, weights_path, 0)
    meta = load_meta(data_path)
    # r = detect(net, meta, img, thresh=thresh)

    custom_detect(net, meta, imgs_list, thresh=thresh, nms=nms, store_root=store_root)
    if rd_flag == True:
        remove_duplicate_detections(imgs_list, store_root, cnms_thresh)


def check_overlap_and_get_max_iou(check_bbox, list_bboxes, min_IoU):
    """
    Returns index of list_bboxes with the maximum overlap with check_bbox. Minimum overlap required is min_IoU.
    Return index -1 if there isn't solution.
    """
    # Lists copies
    list_copy = [list(b) for b in list_bboxes]
    check_bbox_copy = list(check_bbox)

    ind = -1
    maxV = min_IoU

    for i, b in enumerate(list_copy):
    	check_bbox_copy = list(check_bbox)
    	iou = compute_iou(b, check_bbox_copy)
        if iou > maxV:
            ind = i
            maxV = iou

    return ind


def compute_iou(c, a):
    """
    Calculates the intersection over union (IoU) over two bboxes, c and a.

    :param c: control bbox, [x, y, w, h]
    :param a: annotation bbox, [x, y, w, h]
    :return: IoU value
    """
    # Convert w and h to max x, y
    c[3] = c[3] + c[1]
    c[2] = c[2] + c[0]
    a[3] = a[3] + a[1]
    a[2] = a[2] + a[0]

    # intersection
    ixmin = np.maximum(c[0], a[0])
    iymin = np.maximum(c[1], a[1])
    ixmax = np.minimum(c[2], a[2])
    iymax = np.minimum(c[3], a[3])
    iw = np.maximum(ixmax - ixmin + 1., 0.)
    ih = np.maximum(iymax - iymin + 1., 0.)
    inters = iw * ih

    # union
    uni = ((a[2] - a[0] + 1.) * (a[3] - a[1] + 1.) +
           (c[2] - c[0] + 1.) *
           (c[3] - c[1] + 1.) - inters)

    return inters / uni


def format_yolo_bbox(pred_values, imw=None, imh=None):
    """
    Format yolo prediction (relative and central x, y and relative w, h) to normal bbox (x, y top left and absolute w, h)

    :param pred_values: [x, y, w, h] relative and central x, y and relative w, h
    :param imw: image width
    :param imh: image height
    :return: [x, y, w, h] (x, y top left and absolute w, h)
    """

    bbox = [0, 0, 0, 0]
    x, y, w, h = pred_values
    bbox[0] = x - w / 2
    bbox[1] = y - h / 2
    bbox[2] = w
    bbox[3] = h

    if imw is not None and imh is not None:
        bbox[0] = int(bbox[0] * imw)
        bbox[1] = int(bbox[1] * imh)
        bbox[2] = int(bbox[2] * imw)
        bbox[3] = int(bbox[3] * imh)

    return bbox


if __name__ == "__main__":

    weights_list = [9000]
    data_fold_prefix = 'new_data_test'
    data_root_path = '/media/HDD4TB/joanp/tray_images_dataset/'

    test_cfg_file = "yolov3-serunion-test.cfg"
    train_cfg_file = "yolov3-serunion.cfg"

    model_prefix = train_cfg_file.split(".")[0]
    data_type = 'test'

    # Train split folds
    train_folds = [5, 18, 22]
    for i in range(25, 98):	
        if i != 28 and i != 74:	 # folds 28 and 74 contain problems
            train_folds.append(i)

    # Test split folds
    test_folds = []
    for i in range(1, 25):
        if i!=5 and i!=18 and i!=22:
            test_folds.append(i)

    all_folds = train_folds + test_folds

    # Grid search params
    rd_flag = False
    thresh_values = [0.001]  #[0.2]
    nms_values = [0.6]  #[0.4]
    cnms_thresh_values = [0.95]  #[0.95]

    if rd_flag:
        last_preds_folder_name = "preds_9000_best_params"  #'preds_9000_test_multi'  #'preds_9000_test'  #'preds_9000_best_params'
    else:
        last_preds_folder_name = "preds_9000_test_multi"

    # Evaluation params
    min_IoU = 0.5
    max_pred_bboxes = 20 
    images_folder_out = 'images/'
    features_folder_out = 'features/'
    images_suffix = '%s_imgs.txt'
    annotations_folder = 'annotations/'
    annotations_suffix = '%s_labels.txt'
    all_classes_names_path = "/media/HDD4TB/joanp/yolo_classes_new.txt"
    yolo_names = "/media/HDD4TB/joanp/yolo.names"

    # Read all classes names and create classes dictionary    
    with open(all_classes_names_path, 'r') as cn:
        classes_list = cn.readlines()

    dic_classes = {}
    for i in range(len(classes_list)):
        dic_classes[classes_list[i].strip('\n')] = i

    # Add mappings to dic_classes (there are different yolo classes that actually are the same)
    dic_classes['cuscus'] = dic_classes['couscous']
    dic_classes['Canalones'] = dic_classes['canelones']
    dic_classes['canalones'] = dic_classes['canelones']
    dic_classes['ensalada de lentejas '] = dic_classes['ensalada de lentejas']
    dic_classes['fideu\xc3\xa1 '] = dic_classes['fideu\xc3\xa1']
    dic_classes['arroz bolonesa'] = dic_classes['arroz a la bolo\xc3\xb1esa']
    dic_classes['arroz con bolonesa'] = dic_classes['arroz a la bolo\xc3\xb1esa']
    dic_classes['Escalopa de ternera'] = dic_classes['escalopa de ternera']
    dic_classes['escalopa de ternera '] = dic_classes['escalopa de ternera']
    dic_classes['Bistec de ternera a la pimienta'] = dic_classes['bistec de ternera a la pimienta']
    dic_classes['ensalada de cuscus'] = dic_classes['ensalada de couscous']
    dic_classes['espinacas a la crema '] = dic_classes['espinacas a la crema']
    dic_classes['empanadillas '] = dic_classes['empanadillas']
    dic_classes['espaguetis napolitana'] = dic_classes['espaguetis']
    dic_classes['tallarines al pesto'] = dic_classes['espaguetis']


    with open(yolo_names, 'r') as g:
        classes_list = g.readlines()
        yolo_classes_list = [c.strip('\n') for c in classes_list]
        g.close()

    comb_list = list(itertools.product(cnms_thresh_values, nms_values, thresh_values))

    with open("/media/HDD4TB/joanp/gridsearch_baseline.txt", "w") as gs_f:
        gs_f.write("YOLO BASELINE GRID SEARCH\n\n")
        gs_f.close()

    with open("/media/HDD4TB/joanp/gridsearch_baseline.txt", "a") as gs_f:
        gs_f.write("Thresh, NMS, CNMS  -->  Precision, Recall, F1, Percentage of FN \n")
        gs_f.close()

    num_gt = 0
    grid_count = 0
    for cnms_thresh, nms, thresh in comb_list:
        grid_count += 1
        print "Grid Search " + str(grid_count) + "/" + str(len(comb_list)) + "\n"

        pred_values_list = []
        gt_values_list = []
        total_counts = 0

        # For running prediction on all the testing data folds
        for fold in test_folds: #all_folds or test_folds:

            print "Fold " + str(fold)


            data_name = data_fold_prefix + str(fold)
            model_name = model_prefix + '_9000.weights'
            data_path = data_root_path + data_name + "/train_cfg/" + data_name + ".data"


            pred_path = data_root_path + '/' + data_name + '/prediction'
            pred_9000_path = pred_path + '/' + last_preds_folder_name

            if not os.path.isdir(pred_path):
                os.makedirs(pred_path)
            if not os.path.isdir(pred_9000_path):
                os.makedirs(pred_9000_path)

            cfg_path = data_root_path + data_name + '/train_cfg/' + test_cfg_file
            # cfg_path = "/media/HDD3TB/datasets/FoodImageNet/"+data_name+"/train_cfg_old/yolov2-serunion-test.cfg"

            weights_path = data_root_path + data_name + "/backups_" + data_name + "/" + model_name
            # weights_path = "/media/HDD3TB/datasets/FoodImageNet/"+data_name+"/backups_"+data_name+"_unimib-based/"+model_name

            imgs_list = data_root_path + data_name + "/labels/" + data_type + "_imgs_list.txt"
            store_root = pred_path + '/' + last_preds_folder_name

            if weights_path:
                queue = Queue()
                p = Process(target=detect_fold, args=(cfg_path, weights_path, data_path, thresh, nms, store_root))
                p.start()
                p.join()
                p.terminate()


            # Evaluation
            n = fold
            counts = 0
            folder_name = "new_data_test" + str(n)

            if os.path.isdir(data_root_path + folder_name):
                path = data_root_path + folder_name + "/images/test/"
                labels_path = data_root_path + folder_name + "/labels/test/"
                preds_path = "/media/HDD4TB/joanp/tray_images_dataset/new_data_test" + str(n) + "/prediction/" + last_preds_folder_name + '/'
                onlyfiles = [f for f in listdir(path) if isfile(join(path, f))]

                if os.path.isdir(data_root_path + folder_name + "/train_cfg/"):
                    if os.path.isfile(data_root_path + folder_name + "/train_cfg/new_data_test" + str(n) + ".names"):
                        with open(data_root_path + folder_name + "/train_cfg/new_data_test" + str(n) + ".names", 'r') as g:
                            classes_list = g.readlines()
                            classes_list = [c.strip('\n') for c in classes_list]

                        for f in onlyfiles:
                            f_id = f[:-4]

                            # Save image in set directory 
                            # Read RGB image 
                            img = cv2.imread(path + f)

                            #print(img.shape) # (768, 1024, 3)
                            img_h = img.shape[0]
                            img_w = img.shape[1]

                            with open(labels_path + f_id + ".txt", 'r') as f, open(preds_path + f_id + ".txt", 'r') as g:
                                gt_content = f.readlines()
                                preds_content = g.readlines()
                                f.close()
                                g.close()

                            gt_labels = []
                            gt_bboxes = []
                            pred_labels = []
                            pred_bboxes = []
                            pred_probs = []

                            if len(gt_content) > 0:
                                for l in gt_content:
                                    l = l.split()
                                    gt_labels.append(dic_classes[classes_list[int(l[0])]])
                                    gt_bboxes.append([int(float(l[1])*img_w), int(float(l[2])*img_h), int(float(l[3])*img_w), int(float(l[4])*img_h)])

                            if len(preds_content) > 0:
                                for l in preds_content:
                                    l = l.split()
                                    if float(l[1])<0 or float(l[2])<0 or float(l[3])<0 or float(l[4])<0:
                                        print "pred with negative values, discarded!"
                                    else:
                                        pred_labels.append(int(l[0]))
                                        pred_bboxes.append([int(float(l[1])*img_w), int(float(l[2])*img_h), int(float(l[3])*img_w), int(float(l[4])*img_h)])								
                                        pred_probs.append(float(l[5]))

                            num_gts = len(gt_labels)
                            num_preds = len(pred_labels)
                            if grid_count == 1:
                                num_gt += num_gts
                            
                            #if num_preds > 20:
                            #    num_preds = 20

                            for i in range(num_preds):
                                # We assume that preds are sorted by prob

                                # Get index of best GT 
                                ind = check_overlap_and_get_max_iou(pred_bboxes[i], gt_bboxes, min_IoU)

                                if ind == -1: # There isn't a GT bbox and label that matches the prediction, so we set the GT of this pred to background
                                    gt_values_list.append(0)
                                
                                else:  # Exists a GT bbox and label that matches the prediction
                                    gt_values_list.append(gt_labels[ind])

                                    # Remove gt label and bbox from list
                                    del gt_labels[ind]
                                    del gt_bboxes[ind]

                                # Save pred label and img
                                pred_values_list.append(pred_labels[i])
                                

                            if len(gt_labels) > 0:    
                                counts += len(gt_labels)
                                for a in gt_labels:
                                    gt_values_list.append(a)
                                    pred_values_list.append(0)

                print "Fold " + str(n) + " finished!\n"

        total_counts += counts  
        percentage_fns = round(float(total_counts * 100) / float(num_gt), 4)
        print "percentage = " + str(percentage_fns) + "%"

        # Metrics
        y_gt = np.asarray(gt_values_list)
        y_pred = np.asarray(pred_values_list)

        # Compute Precision, Recall and F1 score
        precision, recall, f1_classes, support = sklearn_metrics.precision_recall_fscore_support(y_gt, y_pred, average=None)

        # Don't use background for metrics computation
        support = support[1:]
        precision = precision[1:]
        recall = recall[1:]

        # Precision and recall arrays, only of elements with non zero support
        precision_ = np.nan_to_num(np.take(precision, np.nonzero(support)))
        recall_ = np.nan_to_num(np.take(recall, np.nonzero(support)))

        # F1-score per class
        f1_ = 2*precision_*recall_ / (precision_+recall_)
        f1_ = np.nan_to_num(f1_)  # It is possible to divide by 0 and get nan values. So we set those to 0.0

        # Get mean metrics between all classes
        precision = np.mean(precision_)
        recall = np.mean(recall_)
        f1 = np.mean(f1_)

        with open("/media/HDD4TB/joanp/gridsearch_baseline.txt", "a") as gs_f:
            gs_f.write(str(thresh) + ", " + str(nms) + ", " + str(cnms_thresh) + "  -->  " + str(round(precision,4)) + ", " + str(round(recall,4)) + ", " + str(round(f1,4)) + ", " + str(percentage_fns) + "%" + "\n")
            gs_f.close()

        with codecs.open(all_classes_names_path, 'r', 'utf-8') as f:
            lines = f.readlines()
            classes_names = [l.strip('\n') for l in lines]
            f.close()

        report = sklearn_metrics.classification_report(y_gt, y_pred, target_names=classes_names)

        with codecs.open("/media/HDD4TB/joanp/classification_report.txt", 'w', 'utf-8') as f:
            f.write(report)
            f.close()

        with open("/media/HDD4TB/joanp/f1-score_classes.txt", 'w') as f:
            for v in f1_classes:
                f.write(str(v)+'\n')
            f.close()

        print "YOLO data metrics for params"
        print " -> Threshold = " + str(thresh)
        print " -> NMS = " + str(nms)
        print " -> CNMS = " + str(cnms_thresh)
        print
        print "precision = " + str(precision)
        print "recall = " + str(recall)
        print "f1 = " + str(f1)         
        print
        print "-----------------------------------------------------"
        print  

