from ctypes import *
import os
import math
import random
import numpy as np
import sys
from subprocess import call, Popen
from multiprocessing import Process, Queue


def sample(probs):
    s = sum(probs)
    probs = [a / s for a in probs]
    r = random.uniform(0, 1)
    for i in range(len(probs)):
        r = r - probs[i]
        if r <= 0:
            return i
    return len(probs) - 1


def c_array(ctype, values):
    arr = (ctype * len(values))()
    arr[:] = values
    return arr


class BOX(Structure):
    _fields_ = [("x", c_float),
                ("y", c_float),
                ("w", c_float),
                ("h", c_float)]


class DETECTION(Structure):
    _fields_ = [("bbox", BOX),
                ("classes", c_int),
                ("prob", POINTER(c_float)),
                ("mask", POINTER(c_float)),
                ("objectness", c_float),
                ("sort_class", c_int)]


class IMAGE(Structure):
    _fields_ = [("w", c_int),
                ("h", c_int),
                ("c", c_int),
                ("data", POINTER(c_float))]


class METADATA(Structure):
    _fields_ = [("classes", c_int),
                ("names", POINTER(c_char_p))]


lib = CDLL("/media/HDD3TB/pritomrit/YOLO_weights/libdarknet.so",
           RTLD_GLOBAL)
# lib = CDLL("libdarknet.so", RTLD_GLOBAL)
lib.network_width.argtypes = [c_void_p]
lib.network_width.restype = c_int
lib.network_height.argtypes = [c_void_p]
lib.network_height.restype = c_int

predict = lib.network_predict
predict.argtypes = [c_void_p, POINTER(c_float)]
predict.restype = POINTER(c_float)

set_gpu = lib.cuda_set_device
set_gpu.argtypes = [c_int]

make_image = lib.make_image
make_image.argtypes = [c_int, c_int, c_int]
make_image.restype = IMAGE

get_network_boxes = lib.get_network_boxes
get_network_boxes.argtypes = [c_void_p, c_int, c_int, c_float, c_float, POINTER(c_int), c_int, POINTER(c_int)]
get_network_boxes.restype = POINTER(DETECTION)

make_network_boxes = lib.make_network_boxes
make_network_boxes.argtypes = [c_void_p]
make_network_boxes.restype = POINTER(DETECTION)

free_detections = lib.free_detections
free_detections.argtypes = [POINTER(DETECTION), c_int]

free_ptrs = lib.free_ptrs
free_ptrs.argtypes = [POINTER(c_void_p), c_int]

network_predict = lib.network_predict
network_predict.argtypes = [c_void_p, POINTER(c_float)]

reset_rnn = lib.reset_rnn
reset_rnn.argtypes = [c_void_p]

load_net = lib.load_network
load_net.argtypes = [c_char_p, c_char_p, c_int]
load_net.restype = c_void_p

do_nms_obj = lib.do_nms_obj
do_nms_obj.argtypes = [POINTER(DETECTION), c_int, c_int, c_float]

do_nms_sort = lib.do_nms_sort
do_nms_sort.argtypes = [POINTER(DETECTION), c_int, c_int, c_float]

free_image = lib.free_image
free_network = lib.free_network
# free_network.argtypes = POINTER(NETWORK)
free_image.argtypes = [IMAGE]

letterbox_image = lib.letterbox_image
letterbox_image.argtypes = [IMAGE, c_int, c_int]
letterbox_image.restype = IMAGE

load_meta = lib.get_metadata
lib.get_metadata.argtypes = [c_char_p]
lib.get_metadata.restype = METADATA

load_image = lib.load_image_color
load_image.argtypes = [c_char_p, c_int, c_int]
load_image.restype = IMAGE

rgbgr_image = lib.rgbgr_image
rgbgr_image.argtypes = [IMAGE]

predict_image = lib.network_predict_image
predict_image.argtypes = [c_void_p, IMAGE]
predict_image.restype = POINTER(c_float)


def classify(net, meta, im):
    out = predict_image(net, im)
    res = []
    for i in range(meta.classes):
        res.append((meta.names[i], out[i]))
    res = sorted(res, key=lambda x: -x[1])
    return res


def detect(net, meta, image, thresh=.5, hier_thresh=.5, nms=.45):
    im = load_image(image, 0, 0)
    num = c_int(0)
    pnum = pointer(num)
    predict_image(net, im)
    dets = get_network_boxes(net, im.w, im.h, thresh, hier_thresh, None, 0, pnum)
    num = pnum[0]
    if (nms): do_nms_obj(dets, num, meta.classes, nms);

    res = []
    for j in range(num):
        for i in range(meta.classes):
            if dets[j].prob[i] > 0:
                b = dets[j].bbox
                res.append((meta.names[i], dets[j].prob[i], (b.x, b.y, b.w, b.h)))
    res = sorted(res, key=lambda x: -x[1])
    free_image(im)
    free_detections(dets, num)
    return res


def is_bb_overlap(boxA, boxB):
    isOverlap = True

    minXA, maxXA = boxA[0] - boxA[2] * .5, boxA[0] + boxA[2] * .5
    minYA, maxYA = boxA[1] - boxA[3] * .5, boxA[1] + boxA[3] * .5

    minXB, maxXB = boxB[0] - boxB[2] * .5, boxB[0] + boxB[2] * .5
    minYB, maxYB = boxB[1] - boxB[3] * .5, boxB[1] + boxB[3] * .5

    if (minXB > maxXA or maxXB < minXA or minYB > maxYA or maxYB < minYA):
        isOverlap = False

    return isOverlap


def intersection_over_minbbox(boxA, boxB):
    # determine the (x, y)-coordinates of the intersection rectangle
    xA = max(boxA[0] - boxA[2] * .5, boxB[0] - boxB[2] * .5)
    yA = max(boxA[1] - boxA[3] * .5, boxB[1] - boxB[3] * .5)
    xB = min(boxA[0] + boxA[2] * .5, boxB[0] + boxB[2] * .5)
    yB = min(boxA[1] + boxA[3] * .5, boxB[1] + boxB[3] * .5)

    # compute the area of intersection rectangle
    interArea = abs(xB - xA) * abs(yB - yA)

    # compute the area of both the prediction and ground-truth
    # rectangles
    boxAArea = boxA[2] * boxA[3]
    boxBArea = boxB[2] * boxB[3]

    boxArea = np.minimum(boxAArea, boxBArea)
    # compute the intersection over union by taking the intersection
    # area and dividing it by the sum of prediction + ground-truth
    # areas - the interesection area
    intersection = interArea / float(boxArea)

    # return the intersection over union value
    return intersection


def custom_detect(net, meta, imgs_list, thresh=.5, hier_thresh=.5, nms=.45, store_root=""):
    with open(imgs_list, "r") as f:
        for line in f:
            image = line.strip()
            #print
            image
            img_name = image.split("/")[-1].split(".")[0]
            im = load_image(image, 0, 0)
            num = c_int(0)
            pnum = pointer(num)
            predict_image(net, im)
            dets = get_network_boxes(net, im.w, im.h, thresh, hier_thresh, None, 0, pnum)
            num = pnum[0]
            if (nms): do_nms_obj(dets, num, meta.classes, nms);

            w, h = im.w, im.h
            res = []
            for j in range(num):
                for i in range(meta.classes):
                    if dets[j].prob[i] > 0:
                        b = dets[j].bbox
                        str_dets = " ".join([str(i), str(b.x / w), str(b.y / h), str(b.w / w), str(b.h / h),
                                             str(dets[j].prob[i])]) + "\n"
                        res.append(str_dets)
            #print
            np.shape(res)
            with open(store_root + "/" + img_name + ".txt", "w") as f2:
                f2.writelines(res)
            # res = sorted(res, key=lambda x: -x[1])
            free_image(im)
            free_detections(dets, num)
    # return res


def remove_duplicate_detections(imgs_list, store_root, nms_thresh):
    names_f = '/media/HDD4TB/joanp/yolo.names'
    ind2names = {}
    with open(names_f, 'r') as names_file:
        for i, line in enumerate(names_file):
            ind2names[str(i)] = line.strip('\n')
    
    #print ind2names

    with open(imgs_list, "r") as f:
        for line in f:
            image = line.strip()
            #print
            image
            img_name = image.split("/")[-1].split(".")[0]
            img_preds = store_root + "/" + img_name  + ".txt"

            p_f = open(img_preds, "r")
            p_list = [line.strip() for line in p_f]
            p_f.close()

            lbl_pred_list = []
            bbs_pred = []
            prob_pred = []

            for pred in p_list:
                lbl, x, y, bw, bh, prob = pred.split(" ")
                lbl_pred_list.append(lbl)
                boxA = [float(x), float(y), float(bw), float(bh)]
                bbs_pred.append(boxA)
                prob_pred.append(float(prob))

            aux_lbl_pred_list = lbl_pred_list
            aux_bbs_pred = bbs_pred
            aux_prob_pred = prob_pred

            sort1 = np.argsort(aux_prob_pred)[::-1]
            aux_prob_pred = list(np.array(aux_prob_pred)[sort1])
            aux_lbl_pred_list = list(np.array(aux_lbl_pred_list)[sort1])
            aux_bbs_pred = list(np.array(aux_bbs_pred)[sort1])
            selected_bbox = []
            selected_label = []
            init_index = 0
            for k in range(len(aux_lbl_pred_list) - 1):
                if aux_lbl_pred_list[k] == None:
                    continue
                selected_bbox.append(aux_bbs_pred[k])
                selected_label.append(int(aux_lbl_pred_list[k]))
                init_index = k + 1
                break
            for i in range(init_index, len(aux_lbl_pred_list)):
                if aux_lbl_pred_list[i] == None:
                    continue
                boxB = aux_bbs_pred[i]
                lblB = int(aux_lbl_pred_list[i])
                flag = False
                for j in range(len(selected_bbox)):
                    boxA = selected_bbox[j]
                    lblA = int(selected_label[j])
                    if is_bb_overlap(boxA, boxB) and \
                            ((intersection_over_minbbox(boxA, boxB) > nms_thresh and lblA != lblB)):
                        aux_lbl_pred_list[i] = None
                        aux_bbs_pred[i] = None
                        aux_prob_pred[i] = None
                        flag = True
                        break
                if not flag:
                    selected_bbox.append(boxB)
                    selected_label.append(lblB)

            lbl_pred_list = []
            bbs_pred = []
            prob_pred = []
            for j in range(len(aux_lbl_pred_list)):
                if aux_lbl_pred_list[j] != None:
                    lbl_pred_list.append(aux_lbl_pred_list[j])
                    bbs_pred.append(aux_bbs_pred[j])
                    prob_pred.append(aux_prob_pred[j])

            res = []
            for k in range(len(prob_pred)):
                lbl = lbl_pred_list[k]
                bbs = bbs_pred[k]
                prob = prob_pred[k]
                str_dets = " ".join([str(lbl), str(bbs[0]), str(bbs[1]), str(bbs[2]), str(bbs[3]), str(prob)]) + "\n"
                res.append(str_dets)
            with open(store_root + "/" + img_name + ".txt", "w") as f2:
                f2.writelines(res)

            #print "------------------>", np.shape(prob_pred)


def convert_format():
    pass


def detect_fold(cfg_path, weights_path, data_path, thresh, nms, store_root):
    net = load_net(cfg_path, weights_path, 0)
    meta = load_meta(data_path)
    # r = detect(net, meta, img, thresh=thresh)
    print "num classes"
    print meta.classes
    custom_detect(net, meta, imgs_list, thresh=thresh, nms=nms, store_root=store_root)
    if rd_flag == True:
        remove_duplicate_detections(imgs_list, store_root, cnms_thresh)


if __name__ == "__main__":

    weights_list = [9000]
    data_fold_prefix = 'new_data_test'
    data_root_path = '/media/HDD4TB/joanp/tray_images_dataset/'
    total_data_fold = 97

    test_cfg_file = "yolov3-serunion-test.cfg"
    train_cfg_file = "yolov3-serunion.cfg"

    model_prefix = train_cfg_file.split(".")[0]
    data_type = 'test'
    if len(sys.argv) > 1:
        num_days = int(sys.argv[1])
    else:
        num_days = total_data_fold

    # For running prediction on specific fold
    if num_days == 1:

        for i in weights_list:

            data_name = data_fold_prefix + str(sys.argv[2])

            model_name = model_prefix + '_' + str(i) + '.weights'
            data_path = data_root_path + data_name + "/train_cfg/" + data_name + ".data"

            prediction_path = 'prediction'
            pred_9000_path = 'preds_9000_test'

            pred_path = data_root_path + '/' + data_name + '/' + prediction_path

            pred_9000_path = pred_path + '/' + pred_9000_path

            if not os.path.isdir(pred_path):
                os.makedirs(pred_path)
            if not os.path.isdir(pred_9000_path):
                os.makedirs(pred_9000_path)

            cfg_path = data_root_path + data_name + "/train_cfg/" + test_cfg_file
            # cfg_path = "/media/HDD3TB/datasets/FoodImageNet/"+data_name+"/train_cfg_old/yolov2-serunion-test.cfg"

            weights_path = data_root_path + data_name + "/backups_" + data_name + "/" + model_name
            # weights_path = "/media/HDD3TB/datasets/FoodImageNet/"+data_name+"/backups_"+data_name+"_unimib-based/"+model_name

            rd_flag = True
            cnms_thresh = 0.9
            nms = 0.5
            thresh = 0.001
            imgs_list = data_root_path + data_name + "/labels/" + data_type + "_imgs_list.txt"
            store_root = pred_path + "/preds_" + str(i) + "_" + data_type

            if weights_path:
                queue = Queue()
                p = Process(target=detect_fold, args=(cfg_path, weights_path, data_path, thresh, nms, store_root))
                p.start()
                p.join()
                p.terminate()

    else:
        # For running prediction on all the testing data folds
        for fold in range(0, num_days):

            if fold+1 != 28 or fold+1 != 74:

                total_folds = 0

                for i in weights_list:

                    data_name = data_fold_prefix + str(fold + 1)
                    model_name = model_prefix + '_' + str(i) + '.weights'
                    data_path = data_root_path + data_name + "/train_cfg/" + data_name + ".data"

                    prediction_path = 'prediction'
                    pred_9000_path = 'preds_9000_test'

                    pred_path = data_root_path + '/' + data_name + '/' + prediction_path

                    pred_9000_path = pred_path + '/' + pred_9000_path

                    if not os.path.isdir(pred_path):
                        os.makedirs(pred_path)
                    if not os.path.isdir(pred_9000_path):
                        os.makedirs(pred_9000_path)

                    cfg_path = data_root_path + data_name + '/train_cfg/' + test_cfg_file
                    # cfg_path = "/media/HDD3TB/datasets/FoodImageNet/"+data_name+"/train_cfg_old/yolov2-serunion-test.cfg"

                    weights_path = data_root_path + data_name + "/backups_" + data_name + "/" + model_name
                    # weights_path = "/media/HDD3TB/datasets/FoodImageNet/"+data_name+"/backups_"+data_name+"_unimib-based/"+model_name

                    rd_flag = True
                    cnms_thresh = 0.9
                    nms = 0.5
                    thresh = 0.001
                    imgs_list = data_root_path + data_name + "/labels/" + data_type + "_imgs_list.txt"
                    store_root = pred_path + "/preds_" + str(i) + "_" + data_type

                    if weights_path:
                        queue = Queue()
                        p = Process(target=detect_fold, args=(cfg_path, weights_path, data_path, thresh, nms, store_root))
                        p.start()
                        p.join()
                        p.terminate()
                        total_folds = total_folds + 1
                
                print "------> Fold " + str(fold+1) + " finished! "

