#from training_wrapper.custom_losses import uncertainty_categorical_crossentropy, uncertainty_binary_crossentropy, regularizer_mtl_loss,deactivate_loss
#from keras.layers import uncertainty_categorical_crossentropy, uncertainty_binary_crossentropy, deactivate_loss

def load_parameters():
    """
        Loads the defined parameters
    """

    ALL_MODELS = {
                        # --- BEST FUSION MODEL ---

                        ###################
                        # Model fusion: (FV+YP) * (C+M)
                        ###################
                        'FV_YP_mult_C_M_model': {
                            # Model params
                            'MODEL_TYPE': 'FV_YP_mult_C_M_model',  # see predefined models in models.py
                            'MODEL_NAME': '(FV+YP) * (C+M) MODEL', # 'TFG_VIDEO'
                            'STORE_PATH': '%s/FV_YP_mult_C_M_model',  # insert MODELS_ROOT_PATH in %s. '%s/tfg_video'
                            'MODELS_ROOT_PATH': '/models',
                            'EVALUATE': True,
                            # Dataset params
                            'DATA_ROOT_PATH': '/data_yolo',  # '/data' for single images and '/data_video' for group images
                            'DATASET_NAME': 'Food',
                            'REBUILD_DATASET': True,
                            'ALL_CLASSES_NAMES_PATH': '%s/annotations/yolo_classes_new.txt',
                            'YOLO_NAMES': '%s/annotations/yolo.names',
                            'TRAIN_GT_LABELS': '%s/annotations/train_gt_labels.txt',
                            'TRAIN_GT_UNIQUE': '%s/annotations/train_gt_unique.txt',
                            'TRAIN_IMGS_LIST': '%s/annotations/train_imgs.txt',
                            'TEST_GT_LABELS': '%s/annotations/test_gt_labels.txt',
                            'TEST_GT_UNIQUE': '%s/annotations/test_gt_unique.txt',
                            'TEST_IMGS_LIST': '%s/annotations/test_imgs.txt',
                            'INPUTS_IDS_DATASET': ['menu', 'features', 'yolo_predictions', 'correlations'],
                            'OUTPUTS_IDS_DATASET': ['output', 'output2'],
                            'INPUTS': {
                                'menu': {
                                    'type': 'text', # 'raw-image' for single images and 'video' for group images
                                    'words_as_classes': True,  # specific type text param
                                    'max_text_len': 35, # specific type text param
                                    'path': 'annotations/%s_menu.txt',
                                    'num_classes': 104,
                                },
                                'features': {
                                    'type': 'video-features', # 'raw-image' for single images and 'video' for group images
                                    'path': 'annotations/%s_features.txt',
                                    'frames_path': 'annotations/%s_frames.txt', # only for type video,
                                    'indices_path': 'annotations/%s_video_indices.txt', # only for type video,
                                    'max_video_len': 20,
                                    'feat_len': 1536,
                                    'activation': 'tanh',
                                    'normalization': ['L2'],  # For video-features only accepted L2 or None
                                    'data_augmentation': ['noise'],  # For video-features only accepted noise, random_selections or None
                                },
                                'yolo_predictions': {
                                    'type': 'text', # 'raw-image' for single images and 'video' for group images
                                    'words_as_classes': True,  # specific type text param
                                    'max_text_len': 20, # specific type text param
                                    'path': 'annotations/%s_pred_labels.txt',
                                },
                                'correlations': {
                                    'type': 'text', # 'raw-image' for single images and 'video' for group images
                                    'words_as_classes': True,  # specific type text param
                                    'max_text_len': 20, # specific type text param
                                    'path': 'annotations/%s_pred_labels.txt',
                                    'num_classes': 88,
                                },
                            },
                            'OUTPUTS': {
                                'output': {
                                    'type': 'text',   # 'categorical' for single images and 'text' for group images
                                    'words_as_classes': True,  # specific type text param
                                    'max_text_len': 20, # specific type text param
                                    'path': 'annotations/%s_gt_labels.txt',
                                    'classes': 'annotations/yolo_classes_new.txt',
                                    'metrics': ['multiclass_metrics_custom'],  # multilabel_metrics or multiclass_metrics
                                    'write_type': 'listoflists',  # listoflists or list
                                    'activation': 'softmax',  # sigmoid or softmax
                                    'loss': 'categorical_crossentropy',  # binary_crossentropy or categorical_crossentropy
                                    'save_classes_report': False,  # whether or not to save the classification report for each class
                                },
                                'output2': {
                                    'type': 'text',   # 'categorical' for single images and 'text' for group images
                                    'words_as_classes': True,  # specific type text param
                                    'max_text_len': 20, # specific type text param
                                    'path': 'annotations/%s_gt_labels.txt',
                                    'classes': 'annotations/yolo_classes_new.txt',
                                    'metrics': ['multiclass_metrics_custom'],  # multilabel_metrics or multiclass_metrics
                                    'write_type': 'listoflists',  # listoflists or list
                                    'activation': 'softmax',  # sigmoid or softmax
                                    'loss': 'categorical_crossentropy',  # binary_crossentropy or categorical_crossentropy
                                    'save_classes_report': False,  # whether or not to save the classification report for each class
                                },
                            },
                            # Training params
                            'SPLITS': ['test', 'train'], # ['train','val','test']
                            'EMPTY_LABEL': False,
                            'RELOAD': 0,  #30, #39
                            'PATIENCE': 5,
                            'STOP_METRIC': 'accuracy',
                            'BATCH_SIZE': 24,
                            'LR_DECAY': None,  # number of minimum number of epochs before the next LR decay (set to None to disable)
                            'LR_GAMMA': 0.5,  # multiplier used for decreasing the LR
                            'SHUFFLE': True,
                            'MAX_FRAME': 20,
                            'EMBEDDING_SIZE': 50,
                            'FUSION_INPUT_LEN': 200,
                            'DROPOUT': 0.2, 
                            'WEIGHT_DECAY': 1e-3,  # L2 regularization
                            'RECURRENT_WEIGHT_DECAY': 0.0,  # L2 regularization in recurrent layers
                            'USE_RECURRENT_DROPOUT': False,  # Use dropout in recurrent layers # DANGEROUS!
                            'RECURRENT_DROPOUT_P': 0.5,  # Percentage of units to drop in recurrent layers
                            'INIT_LAYERS': ['tanh'],
                            'DECODER_HIDDEN_SIZE': 484,  # For models with LSTM decoder (ABiViRNet 484)
                            'TARGET_TEXT_EMBEDDING_SIZE': 99,
                            'ADDITIONAL_OUTPUT_MERGE_MODE': 'sum',
                            'CLASSIFIER_ACTIVATION': 'softmax',
                            'EPSILON': 1e-7,
                            'VOCABULARY_SIZE': 104,
                            'MIN_OCCURRENCES_VOCAB': 0,

                            # Prediction params
                            'SAMPLING_SAVE_MODE': 'numpy',
                            'SAMPLING_RELOAD_POINT': 30,
                            'SAMPLING_RELOAD_EPOCH': True,
                            'TOKENIZATION_METHOD': None,
                            'SAMPLING': 'max_likelihood',  # Possible values: multinomial or max_likelihood (recommended)
                            'TEMPERATURE': 1,
                        },
                        

                        # --- CNN and GENERATE FEATURES models ---

                        ###################
                        # GENERATE FEATURES model  -->  uses the CNN model to generate feature vectors for the fusion models
                        ###################
                        'features': {
                            # Model params
                            'MODEL_TYPE': 'InceptionResNetV2_PlusFC',  # see predefined models in models.py
                            'MODEL_NAME': 'TFG_FEATURES', # 'TFG_VIDEO'
                            'STORE_PATH': '%s/InceptionResNetV2_PlusFC',  # insert MODELS_ROOT_PATH in %s. '%s/tfg_video'
                            'MODELS_ROOT_PATH': '/models',
                            'EVALUATE': True,
                            # Dataset params
                            'DATA_ROOT_PATH': '/data_yolo',  # '/data' for single images and '/data_video' for group images
                            'SPLITS': ['test', 'train'],
                            'DATASET_NAME': 'Food',
                            'REBUILD_DATASET': True,
                            'INPUTS_IDS_DATASET': ['images'],
                            'OUTPUTS_IDS_DATASET': None,
                            'INPUTS': {
                                'images': {
                                    'type': 'raw-image', # 'raw-image' for single images and 'video' for group images
                                    'path': 'annotations/%s_imgs.txt',
                                    'frames_path': 'annotations/%s_frames.txt',
                                    'img_size': [342, 342, 3],
                                    'img_size_crop': [299, 299, 3],
                                },
                            },
                            'OUTPUTS': {
                            },
                            # Prediction params
                            'SAMPLING_SAVE_MODE': 'numpy',
                            'SAMPLING_RELOAD_POINT': 31,
                            'SAMPLING_RELOAD_EPOCH': True,
                            'TOKENIZATION_METHOD': None,
                            'SAMPLING': 'max_likelihood',  # Possible values: multinomial or max_likelihood (recommended)
                            'TEMPERATURE': 1,
                        },

                        ###################
                        # CNN model  -->  InceptionResNetV2_PlusFC
                        ###################
                        'CNN': {
                            # Model params
                            'MODEL_TYPE': 'InceptionResNetV2_PlusFC',  # see predefined models in models.py
                            'MODEL_NAME': 'InceptionResNetV2_PlusFC', # 'TFG_VIDEO'
                            'STORE_PATH': '%s/InceptionResNetV2_PlusFC',  # insert MODELS_ROOT_PATH in %s. '%s/tfg_video'
                            'MODELS_ROOT_PATH': '/models',
                            'EVALUATE': True,
                            # Dataset params
                            'DATA_ROOT_PATH': '/data_yolo',  # '/data' for single images and '/data_video' for group images
                            'SPLITS': ['test', 'train'],
                            'DATASET_NAME': 'Food',
                            'REBUILD_DATASET': True,
                            'INPUTS_IDS_DATASET': ['images'],
                            'OUTPUTS_IDS_DATASET': ['output'],
                            'INPUTS': {
                                'images': {
                                    'type': 'raw-image', # 'raw-image' for single images and 'video' for group images
                                    'path': 'annotations/%s_imgs.txt',
                                    'img_size': [342, 342, 3],
                                    'img_size_crop': [299, 299, 3],
                                },
                            },
                            'OUTPUTS': {
                                'output': {
                                    'type': 'categorical',   # 'categorical' for single images and 'text' for group images
                                    'path': 'annotations/%s_gt_ind.txt',
                                    'classes': 'annotations/yolo_classes_new.txt',
                                    'metrics': ['multiclass_metrics'],  # multilabel_metrics or multiclass_metrics
                                    'write_type': 'list',  # listoflists or list
                                    'activation': 'softmax',  # sigmoid or softmax
                                    'loss': 'categorical_crossentropy',  # binary_crossentropy or categorical_crossentropy
                                    'save_classes_report': True,  # whether or not to save the classification report for each class
                                },
                            },
                            # Training params
                            'SPLITS': ['test', 'train'], # ['train','val','test']
                            'EMPTY_LABEL': False,
                            'RELOAD': 0,
                            'PATIENCE': 5,
                            'STOP_METRIC': 'accuracy',
                            'MAX_EPOCH': 50,
                            'BATCH_SIZE': 24,
                            'LR_DECAY': None,  # number of minimum number of epochs before the next LR decay (set to None to disable)
                            'LR_GAMMA': 0.5,  # multiplier used for decreasing the LR
                            'SHUFFLE': True,
                            'VOCABULARY_SIZE': 96,
                            'DROPOUT': 0.5,

                            # Prediction params
                            'SAMPLING_SAVE_MODE': 'numpy',
                            'SAMPLING_RELOAD_POINT': 38,
                            'SAMPLING_RELOAD_EPOCH': True,
                            'TOKENIZATION_METHOD': None,
                            'SAMPLING': 'max_likelihood',  # Possible values: multinomial or max_likelihood (recommended)
                            'TEMPERATURE': 1,
                        },


                        # --- OTHER TESTED FUSION MODELS ---

                        ###################
                        # Model fusion: (FV+YP) concat (C+M)
                        ###################
                        'FV_YP_concat_C_M_model': {
                            # Model params
                            'MODEL_TYPE': 'FV_YP_concat_C_M_model',  # see predefined models in models.py
                            'MODEL_NAME': '(FV+YP) concat (C+M) MODEL', # 'TFG_VIDEO'
                            'STORE_PATH': '%s/FV_YP_concat_C_M_model',  # insert MODELS_ROOT_PATH in %s. '%s/tfg_video'
                            'MODELS_ROOT_PATH': '/models',
                            'EVALUATE': True,
                            # Dataset params
                            'DATA_ROOT_PATH': '/data_yolo',  # '/data' for single images and '/data_video' for group images
                            'DATASET_NAME': 'Food',
                            'REBUILD_DATASET': True,
                            'ALL_CLASSES_NAMES_PATH': '%s/annotations/yolo_classes_new.txt',
                            'YOLO_NAMES': '%s/annotations/yolo.names',
                            'TRAIN_GT_LABELS': '%s/annotations/train_gt_labels.txt',
                            'TRAIN_GT_UNIQUE': '%s/annotations/train_gt_unique.txt',
                            'TRAIN_IMGS_LIST': '%s/annotations/train_imgs.txt',
                            'TEST_GT_LABELS': '%s/annotations/test_gt_labels.txt',
                            'TEST_GT_UNIQUE': '%s/annotations/test_gt_unique.txt',
                            'TEST_IMGS_LIST': '%s/annotations/test_imgs.txt',
                            'INPUTS_IDS_DATASET': ['menu', 'features', 'yolo_predictions', 'correlations'],
                            'OUTPUTS_IDS_DATASET': ['output', 'output2'],
                            'INPUTS': {
                                'menu': {
                                    'type': 'text', # 'raw-image' for single images and 'video' for group images
                                    'words_as_classes': True,  # specific type text param
                                    'max_text_len': 35, # specific type text param
                                    'path': 'annotations/%s_menu.txt',
                                    'num_classes': 104,
                                },
                                'features': {
                                    'type': 'video-features', # 'raw-image' for single images and 'video' for group images
                                    'path': 'annotations/%s_features.txt',
                                    'frames_path': 'annotations/%s_frames.txt', # only for type video,
                                    'indices_path': 'annotations/%s_video_indices.txt', # only for type video,
                                    'max_video_len': 20,
                                    'feat_len': 1536,
                                    'activation': 'tanh',
                                    'normalization': ['L2'],  # For video-features only accepted L2 or None
                                    'data_augmentation': ['noise'],  # For video-features only accepted noise, random_selections or None
                                },
                                'yolo_predictions': {
                                    'type': 'text', # 'raw-image' for single images and 'video' for group images
                                    'words_as_classes': True,  # specific type text param
                                    'max_text_len': 20, # specific type text param
                                    'path': 'annotations/%s_pred_labels.txt',
                                },
                                'correlations': {
                                    'type': 'text', # 'raw-image' for single images and 'video' for group images
                                    'words_as_classes': True,  # specific type text param
                                    'max_text_len': 20, # specific type text param
                                    'path': 'annotations/%s_pred_labels.txt',
                                    'num_classes': 88,
                                },
                            },
                            'OUTPUTS': {
                                'output': {
                                    'type': 'text',   # 'categorical' for single images and 'text' for group images
                                    'words_as_classes': True,  # specific type text param
                                    'max_text_len': 20, # specific type text param
                                    'path': 'annotations/%s_gt_labels.txt',
                                    'classes': 'annotations/yolo_classes_new.txt',
                                    'metrics': ['multiclass_metrics_custom'],  # multilabel_metrics or multiclass_metrics
                                    'write_type': 'listoflists',  # listoflists or list
                                    'activation': 'softmax',  # sigmoid or softmax
                                    'loss': 'categorical_crossentropy',  # binary_crossentropy or categorical_crossentropy
                                    'save_classes_report': False,  # whether or not to save the classification report for each class
                                },
                                'output2': {
                                    'type': 'text',   # 'categorical' for single images and 'text' for group images
                                    'words_as_classes': True,  # specific type text param
                                    'max_text_len': 20, # specific type text param
                                    'path': 'annotations/%s_gt_labels.txt',
                                    'classes': 'annotations/yolo_classes_new.txt',
                                    'metrics': ['multiclass_metrics_custom'],  # multilabel_metrics or multiclass_metrics
                                    'write_type': 'listoflists',  # listoflists or list
                                    'activation': 'softmax',  # sigmoid or softmax
                                    'loss': 'categorical_crossentropy',  # binary_crossentropy or categorical_crossentropy
                                    'save_classes_report': False,  # whether or not to save the classification report for each class
                                },
                            },
                            # Training params
                            'SPLITS': ['test', 'train'], # ['train','val','test']
                            'EMPTY_LABEL': False,
                            'RELOAD': 0,  #30, #39
                            'PATIENCE': 5,
                            'STOP_METRIC': 'accuracy',
                            'BATCH_SIZE': 24,
                            'LR_DECAY': None,  # number of minimum number of epochs before the next LR decay (set to None to disable)
                            'LR_GAMMA': 0.5,  # multiplier used for decreasing the LR
                            'SHUFFLE': True,
                            'MAX_FRAME': 20,
                            'EMBEDDING_SIZE': 50,
                            'FUSION_INPUT_LEN': 200,
                            'DROPOUT': 0.2, 
                            'WEIGHT_DECAY': 1e-3,  # L2 regularization
                            'RECURRENT_WEIGHT_DECAY': 0.0,  # L2 regularization in recurrent layers
                            'USE_RECURRENT_DROPOUT': False,  # Use dropout in recurrent layers # DANGEROUS!
                            'RECURRENT_DROPOUT_P': 0.5,  # Percentage of units to drop in recurrent layers
                            'INIT_LAYERS': ['tanh'],
                            'DECODER_HIDDEN_SIZE': 484,  # For models with LSTM decoder (ABiViRNet 484)
                            'TARGET_TEXT_EMBEDDING_SIZE': 99,
                            'ADDITIONAL_OUTPUT_MERGE_MODE': 'concat',
                            'CLASSIFIER_ACTIVATION': 'softmax',
                            'EPSILON': 1e-7,
                            'VOCABULARY_SIZE': 104,
                            'MIN_OCCURRENCES_VOCAB': 0,

                            # Prediction params
                            'SAMPLING_SAVE_MODE': 'numpy',
                            'SAMPLING_RELOAD_POINT': 30,
                            'SAMPLING_RELOAD_EPOCH': True,
                            'TOKENIZATION_METHOD': None,
                            'SAMPLING': 'max_likelihood',  # Possible values: multinomial or max_likelihood (recommended)
                            'TEMPERATURE': 1,
                        },

                        ###################
                        # Model fusion: (FV+YP+C) + (C*M)
                        ###################
                        'FV_YP_C_fusion_C_M_mult_model': {
                            # Model params
                            'MODEL_TYPE': 'FV_YP_C_fusion_C_M_mult_model',  # see predefined models in models.py
                            'MODEL_NAME': '(FV+YP+C) + (C*M) MODEL', # 'TFG_VIDEO'
                            'STORE_PATH': '%s/FV_YP_C_fusion_C_M_mult_model',  # insert MODELS_ROOT_PATH in %s. '%s/tfg_video'
                            'MODELS_ROOT_PATH': '/models',
                            'EVALUATE': True,
                            # Dataset params
                            'DATA_ROOT_PATH': '/data_yolo',  # '/data' for single images and '/data_video' for group images
                            'DATASET_NAME': 'Food',
                            'REBUILD_DATASET': True,
                            'ALL_CLASSES_NAMES_PATH': '%s/annotations/yolo_classes_new.txt',
                            'YOLO_NAMES': '%s/annotations/yolo.names',
                            'TRAIN_GT_LABELS': '%s/annotations/train_gt_labels.txt',
                            'TRAIN_GT_UNIQUE': '%s/annotations/train_gt_unique.txt',
                            'TRAIN_IMGS_LIST': '%s/annotations/train_imgs.txt',
                            'TEST_GT_LABELS': '%s/annotations/test_gt_labels.txt',
                            'TEST_GT_UNIQUE': '%s/annotations/test_gt_unique.txt',
                            'TEST_IMGS_LIST': '%s/annotations/test_imgs.txt',
                            'INPUTS_IDS_DATASET': ['menu', 'features', 'yolo_predictions', 'correlations'],
                            'OUTPUTS_IDS_DATASET': ['output', 'output2'],
                            'INPUTS': {
                                'menu': {
                                    'type': 'text', # 'raw-image' for single images and 'video' for group images
                                    'words_as_classes': True,  # specific type text param
                                    'max_text_len': 35, # specific type text param
                                    'path': 'annotations/%s_menu.txt',
                                    'num_classes': 104,
                                },
                                'features': {
                                    'type': 'video-features', # 'raw-image' for single images and 'video' for group images
                                    'path': 'annotations/%s_features.txt',
                                    'frames_path': 'annotations/%s_frames.txt', # only for type video,
                                    'indices_path': 'annotations/%s_video_indices.txt', # only for type video,
                                    'max_video_len': 20,
                                    'feat_len': 1536,
                                    'activation': 'tanh',
                                    'normalization': ['L2'],  # For video-features only accepted L2 or None
                                    'data_augmentation': ['noise'],  # For video-features only accepted noise, random_selections or None
                                },
                                'yolo_predictions': {
                                    'type': 'text', # 'raw-image' for single images and 'video' for group images
                                    'words_as_classes': True,  # specific type text param
                                    'max_text_len': 20, # specific type text param
                                    'path': 'annotations/%s_pred_labels.txt',
                                },
                                'correlations': {
                                    'type': 'text', # 'raw-image' for single images and 'video' for group images
                                    'words_as_classes': True,  # specific type text param
                                    'max_text_len': 20, # specific type text param
                                    'path': 'annotations/%s_pred_labels.txt',
                                    'num_classes': 88,
                                },
                            },
                            'OUTPUTS': {
                                'output': {
                                    'type': 'text',   # 'categorical' for single images and 'text' for group images
                                    'words_as_classes': True,  # specific type text param
                                    'max_text_len': 20, # specific type text param
                                    'path': 'annotations/%s_gt_labels.txt',
                                    'classes': 'annotations/yolo_classes_new.txt',
                                    'metrics': ['multiclass_metrics_custom'],  # multilabel_metrics or multiclass_metrics
                                    'write_type': 'listoflists',  # listoflists or list
                                    'activation': 'softmax',  # sigmoid or softmax
                                    'loss': 'categorical_crossentropy',  # binary_crossentropy or categorical_crossentropy
                                    'save_classes_report': False,  # whether or not to save the classification report for each class
                                },
                                'output2': {
                                    'type': 'text',   # 'categorical' for single images and 'text' for group images
                                    'words_as_classes': True,  # specific type text param
                                    'max_text_len': 20, # specific type text param
                                    'path': 'annotations/%s_gt_labels.txt',
                                    'classes': 'annotations/yolo_classes_new.txt',
                                    'metrics': ['multiclass_metrics_custom'],  # multilabel_metrics or multiclass_metrics
                                    'write_type': 'listoflists',  # listoflists or list
                                    'activation': 'softmax',  # sigmoid or softmax
                                    'loss': 'categorical_crossentropy',  # binary_crossentropy or categorical_crossentropy
                                    'save_classes_report': False,  # whether or not to save the classification report for each class
                                },
                            },
                            # Training params
                            'SPLITS': ['test', 'train'], # ['train','val','test']
                            'EMPTY_LABEL': False,
                            'RELOAD': 0,  #30, #39
                            'PATIENCE': 5,
                            'STOP_METRIC': 'accuracy',
                            'BATCH_SIZE': 24,
                            'LR_DECAY': None,  # number of minimum number of epochs before the next LR decay (set to None to disable)
                            'LR_GAMMA': 0.5,  # multiplier used for decreasing the LR
                            'SHUFFLE': True,
                            'MAX_FRAME': 20,
                            'EMBEDDING_SIZE': 50,
                            'FUSION_INPUT_LEN': 200,
                            'DROPOUT': 0.2, 
                            'WEIGHT_DECAY': 1e-3,  # L2 regularization
                            'RECURRENT_WEIGHT_DECAY': 0.0,  # L2 regularization in recurrent layers
                            'USE_RECURRENT_DROPOUT': False,  # Use dropout in recurrent layers # DANGEROUS!
                            'RECURRENT_DROPOUT_P': 0.5,  # Percentage of units to drop in recurrent layers
                            'INIT_LAYERS': ['tanh'],
                            'DECODER_HIDDEN_SIZE': 484,  # For models with LSTM decoder (ABiViRNet 484)
                            'TARGET_TEXT_EMBEDDING_SIZE': 99,
                            'ADDITIONAL_OUTPUT_MERGE_MODE': 'sum',
                            'CLASSIFIER_ACTIVATION': 'softmax',
                            'EPSILON': 1e-7,
                            'VOCABULARY_SIZE': 104,
                            'MIN_OCCURRENCES_VOCAB': 0,

                            # Prediction params
                            'SAMPLING_SAVE_MODE': 'numpy',
                            'SAMPLING_RELOAD_POINT': 30,
                            'SAMPLING_RELOAD_EPOCH': True,
                            'TOKENIZATION_METHOD': None,
                            'SAMPLING': 'max_likelihood',  # Possible values: multinomial or max_likelihood (recommended)
                            'TEMPERATURE': 1,
                        },

                        ###################
                        # Model fusion: (FV+YP+C) + (C+M)
                        ###################
                        'FV_YP_C_fusion_C_M_model': {
                            # Model params
                            'MODEL_TYPE': 'FV_YP_C_fusion_C_M_model',  # see predefined models in models.py
                            'MODEL_NAME': '(FV+YP+C) + (C+M) MODEL', # 'TFG_VIDEO'
                            'STORE_PATH': '%s/FV_YP_C_fusion_C_M_model',  # insert MODELS_ROOT_PATH in %s. '%s/tfg_video'
                            'MODELS_ROOT_PATH': '/models',
                            'EVALUATE': True,
                            # Dataset params
                            'DATA_ROOT_PATH': '/data_yolo',  # '/data' for single images and '/data_video' for group images
                            'DATASET_NAME': 'Food',
                            'REBUILD_DATASET': True,
                            'ALL_CLASSES_NAMES_PATH': '%s/annotations/yolo_classes_new.txt',
                            'YOLO_NAMES': '%s/annotations/yolo.names',
                            'TRAIN_GT_LABELS': '%s/annotations/train_gt_labels.txt',
                            'TRAIN_GT_UNIQUE': '%s/annotations/train_gt_unique.txt',
                            'TRAIN_IMGS_LIST': '%s/annotations/train_imgs.txt',
                            'TEST_GT_LABELS': '%s/annotations/test_gt_labels.txt',
                            'TEST_GT_UNIQUE': '%s/annotations/test_gt_unique.txt',
                            'TEST_IMGS_LIST': '%s/annotations/test_imgs.txt',
                            'INPUTS_IDS_DATASET': ['menu', 'features', 'yolo_predictions', 'correlations'],
                            'OUTPUTS_IDS_DATASET': ['output', 'output2'],
                            'INPUTS': {
                                'menu': {
                                    'type': 'text', # 'raw-image' for single images and 'video' for group images
                                    'words_as_classes': True,  # specific type text param
                                    'max_text_len': 35, # specific type text param
                                    'path': 'annotations/%s_menu.txt',
                                    'num_classes': 104,
                                },
                                'features': {
                                    'type': 'video-features', # 'raw-image' for single images and 'video' for group images
                                    'path': 'annotations/%s_features.txt',
                                    'frames_path': 'annotations/%s_frames.txt', # only for type video,
                                    'indices_path': 'annotations/%s_video_indices.txt', # only for type video,
                                    'max_video_len': 20,
                                    'feat_len': 1536,
                                    'activation': 'tanh',
                                    'normalization': ['L2'],  # For video-features only accepted L2 or None
                                    'data_augmentation': ['noise'],  # For video-features only accepted noise, random_selections or None
                                },
                                'yolo_predictions': {
                                    'type': 'text', # 'raw-image' for single images and 'video' for group images
                                    'words_as_classes': True,  # specific type text param
                                    'max_text_len': 20, # specific type text param
                                    'path': 'annotations/%s_pred_labels.txt',
                                },
                                'correlations': {
                                    'type': 'text', # 'raw-image' for single images and 'video' for group images
                                    'words_as_classes': True,  # specific type text param
                                    'max_text_len': 20, # specific type text param
                                    'path': 'annotations/%s_pred_labels.txt',
                                    'num_classes': 88,
                                },
                            },
                            'OUTPUTS': {
                                'output': {
                                    'type': 'text',   # 'categorical' for single images and 'text' for group images
                                    'words_as_classes': True,  # specific type text param
                                    'max_text_len': 20, # specific type text param
                                    'path': 'annotations/%s_gt_labels.txt',
                                    'classes': 'annotations/yolo_classes_new.txt',
                                    'metrics': ['multiclass_metrics_custom'],  # multilabel_metrics or multiclass_metrics
                                    'write_type': 'listoflists',  # listoflists or list
                                    'activation': 'softmax',  # sigmoid or softmax
                                    'loss': 'categorical_crossentropy',  # binary_crossentropy or categorical_crossentropy
                                    'save_classes_report': False,  # whether or not to save the classification report for each class
                                },
                                'output2': {
                                    'type': 'text',   # 'categorical' for single images and 'text' for group images
                                    'words_as_classes': True,  # specific type text param
                                    'max_text_len': 20, # specific type text param
                                    'path': 'annotations/%s_gt_labels.txt',
                                    'classes': 'annotations/yolo_classes_new.txt',
                                    'metrics': ['multiclass_metrics_custom'],  # multilabel_metrics or multiclass_metrics
                                    'write_type': 'listoflists',  # listoflists or list
                                    'activation': 'softmax',  # sigmoid or softmax
                                    'loss': 'categorical_crossentropy',  # binary_crossentropy or categorical_crossentropy
                                    'save_classes_report': False,  # whether or not to save the classification report for each class
                                },
                            },
                            # Training params
                            'SPLITS': ['test', 'train'], # ['train','val','test']
                            'EMPTY_LABEL': False,
                            'RELOAD': 0,  #30, #39
                            'PATIENCE': 5,
                            'STOP_METRIC': 'accuracy',
                            'BATCH_SIZE': 24,
                            'LR_DECAY': None,  # number of minimum number of epochs before the next LR decay (set to None to disable)
                            'LR_GAMMA': 0.5,  # multiplier used for decreasing the LR
                            'SHUFFLE': True,
                            'MAX_FRAME': 20,
                            'EMBEDDING_SIZE': 50,
                            'FUSION_INPUT_LEN': 200,
                            'DROPOUT': 0.2, 
                            'WEIGHT_DECAY': 1e-3,  # L2 regularization
                            'RECURRENT_WEIGHT_DECAY': 0.0,  # L2 regularization in recurrent layers
                            'USE_RECURRENT_DROPOUT': False,  # Use dropout in recurrent layers # DANGEROUS!
                            'RECURRENT_DROPOUT_P': 0.5,  # Percentage of units to drop in recurrent layers
                            'INIT_LAYERS': ['tanh'],
                            'DECODER_HIDDEN_SIZE': 484,  # For models with LSTM decoder (ABiViRNet 484)
                            'TARGET_TEXT_EMBEDDING_SIZE': 99,
                            'ADDITIONAL_OUTPUT_MERGE_MODE': 'sum',
                            'CLASSIFIER_ACTIVATION': 'softmax',
                            'EPSILON': 1e-7,
                            'VOCABULARY_SIZE': 104,
                            'MIN_OCCURRENCES_VOCAB': 0,

                            # Prediction params
                            'SAMPLING_SAVE_MODE': 'numpy',
                            'SAMPLING_RELOAD_POINT': 30,
                            'SAMPLING_RELOAD_EPOCH': True,
                            'TOKENIZATION_METHOD': None,
                            'SAMPLING': 'max_likelihood',  # Possible values: multinomial or max_likelihood (recommended)
                            'TEMPERATURE': 1,
                        },

                        ###################
                        # Model fusion: (FV+YP+C) + (C*M)
                        ###################
                        'FV_YP_C_fusion_C_M_mult_model': {
                            # Model params
                            'MODEL_TYPE': 'FV_YP_C_fusion_C_M_mult_model',  # see predefined models in models.py
                            'MODEL_NAME': '(FV+YP+C) + (C*M) MODEL', # 'TFG_VIDEO'
                            'STORE_PATH': '%s/FV_YP_C_fusion_C_M_mult_model',  # insert MODELS_ROOT_PATH in %s. '%s/tfg_video'
                            'MODELS_ROOT_PATH': '/models',
                            'EVALUATE': True,
                            # Dataset params
                            'DATA_ROOT_PATH': '/data_yolo',  # '/data' for single images and '/data_video' for group images
                            'DATASET_NAME': 'Food',
                            'REBUILD_DATASET': True,
                            'ALL_CLASSES_NAMES_PATH': '%s/annotations/yolo_classes_new.txt',
                            'YOLO_NAMES': '%s/annotations/yolo.names',
                            'TRAIN_GT_LABELS': '%s/annotations/train_gt_labels.txt',
                            'TRAIN_GT_UNIQUE': '%s/annotations/train_gt_unique.txt',
                            'TRAIN_IMGS_LIST': '%s/annotations/train_imgs.txt',
                            'TEST_GT_LABELS': '%s/annotations/test_gt_labels.txt',
                            'TEST_GT_UNIQUE': '%s/annotations/test_gt_unique.txt',
                            'TEST_IMGS_LIST': '%s/annotations/test_imgs.txt',
                            'INPUTS_IDS_DATASET': ['menu', 'features', 'yolo_predictions', 'correlations'],
                            'OUTPUTS_IDS_DATASET': ['output', 'output2'],
                            'INPUTS': {
                                'menu': {
                                    'type': 'text', # 'raw-image' for single images and 'video' for group images
                                    'words_as_classes': True,  # specific type text param
                                    'max_text_len': 35, # specific type text param
                                    'path': 'annotations/%s_menu.txt',
                                    'num_classes': 104,
                                },
                                'features': {
                                    'type': 'video-features', # 'raw-image' for single images and 'video' for group images
                                    'path': 'annotations/%s_features.txt',
                                    'frames_path': 'annotations/%s_frames.txt', # only for type video,
                                    'indices_path': 'annotations/%s_video_indices.txt', # only for type video,
                                    'max_video_len': 20,
                                    'feat_len': 1536,
                                    'activation': 'tanh',
                                    'normalization': ['L2'],  # For video-features only accepted L2 or None
                                    'data_augmentation': ['noise'],  # For video-features only accepted noise, random_selections or None
                                },
                                'yolo_predictions': {
                                    'type': 'text', # 'raw-image' for single images and 'video' for group images
                                    'words_as_classes': True,  # specific type text param
                                    'max_text_len': 20, # specific type text param
                                    'path': 'annotations/%s_pred_labels.txt',
                                },
                                'correlations': {
                                    'type': 'text', # 'raw-image' for single images and 'video' for group images
                                    'words_as_classes': True,  # specific type text param
                                    'max_text_len': 20, # specific type text param
                                    'path': 'annotations/%s_pred_labels.txt',
                                    'num_classes': 88,
                                },
                            },
                            'OUTPUTS': {
                                'output': {
                                    'type': 'text',   # 'categorical' for single images and 'text' for group images
                                    'words_as_classes': True,  # specific type text param
                                    'max_text_len': 20, # specific type text param
                                    'path': 'annotations/%s_gt_labels.txt',
                                    'classes': 'annotations/yolo_classes_new.txt',
                                    'metrics': ['multiclass_metrics_custom'],  # multilabel_metrics or multiclass_metrics
                                    'write_type': 'listoflists',  # listoflists or list
                                    'activation': 'softmax',  # sigmoid or softmax
                                    'loss': 'categorical_crossentropy',  # binary_crossentropy or categorical_crossentropy
                                    'save_classes_report': False,  # whether or not to save the classification report for each class
                                },
                                'output2': {
                                    'type': 'text',   # 'categorical' for single images and 'text' for group images
                                    'words_as_classes': True,  # specific type text param
                                    'max_text_len': 20, # specific type text param
                                    'path': 'annotations/%s_gt_labels.txt',
                                    'classes': 'annotations/yolo_classes_new.txt',
                                    'metrics': ['multiclass_metrics_custom'],  # multilabel_metrics or multiclass_metrics
                                    'write_type': 'listoflists',  # listoflists or list
                                    'activation': 'softmax',  # sigmoid or softmax
                                    'loss': 'categorical_crossentropy',  # binary_crossentropy or categorical_crossentropy
                                    'save_classes_report': False,  # whether or not to save the classification report for each class
                                },
                            },
                            # Training params
                            'SPLITS': ['test', 'train'], # ['train','val','test']
                            'EMPTY_LABEL': False,
                            'RELOAD': 0,  #30, #39
                            'PATIENCE': 5,
                            'STOP_METRIC': 'accuracy',
                            'BATCH_SIZE': 24,
                            'LR_DECAY': None,  # number of minimum number of epochs before the next LR decay (set to None to disable)
                            'LR_GAMMA': 0.5,  # multiplier used for decreasing the LR
                            'SHUFFLE': True,
                            'MAX_FRAME': 20,
                            'EMBEDDING_SIZE': 50,
                            'FUSION_INPUT_LEN': 200,
                            'DROPOUT': 0.2, 
                            'WEIGHT_DECAY': 1e-3,  # L2 regularization
                            'RECURRENT_WEIGHT_DECAY': 0.0,  # L2 regularization in recurrent layers
                            'USE_RECURRENT_DROPOUT': False,  # Use dropout in recurrent layers # DANGEROUS!
                            'RECURRENT_DROPOUT_P': 0.5,  # Percentage of units to drop in recurrent layers
                            'INIT_LAYERS': ['tanh'],
                            'DECODER_HIDDEN_SIZE': 484,  # For models with LSTM decoder (ABiViRNet 484)
                            'TARGET_TEXT_EMBEDDING_SIZE': 99,
                            'ADDITIONAL_OUTPUT_MERGE_MODE': 'sum',
                            'CLASSIFIER_ACTIVATION': 'softmax',
                            'EPSILON': 1e-7,
                            'VOCABULARY_SIZE': 104,
                            'MIN_OCCURRENCES_VOCAB': 0,

                            # Prediction params
                            'SAMPLING_SAVE_MODE': 'numpy',
                            'SAMPLING_RELOAD_POINT': 30,
                            'SAMPLING_RELOAD_EPOCH': True,
                            'TOKENIZATION_METHOD': None,
                            'SAMPLING': 'max_likelihood',  # Possible values: multinomial or max_likelihood (recommended)
                            'TEMPERATURE': 1,
                        },

                        ###################
                        # Model fusion: (FV+YP+C) * (C*M)
                        ###################
                        'FV_YP_C_mult_C_M_mult_model': {
                            # Model params
                            'MODEL_TYPE': 'FV_YP_C_mult_C_M_mult_model',  # see predefined models in models.py
                            'MODEL_NAME': '(FV+YP+C) * (C*M) MODEL', # 'TFG_VIDEO'
                            'STORE_PATH': '%s/FV_YP_C_mult_C_M_mult_model',  # insert MODELS_ROOT_PATH in %s. '%s/tfg_video'
                            'MODELS_ROOT_PATH': '/models',
                            'EVALUATE': True,
                            # Dataset params
                            'DATA_ROOT_PATH': '/data_yolo',  # '/data' for single images and '/data_video' for group images
                            'DATASET_NAME': 'Food',
                            'REBUILD_DATASET': True,
                            'ALL_CLASSES_NAMES_PATH': '%s/annotations/yolo_classes_new.txt',
                            'YOLO_NAMES': '%s/annotations/yolo.names',
                            'TRAIN_GT_LABELS': '%s/annotations/train_gt_labels.txt',
                            'TRAIN_GT_UNIQUE': '%s/annotations/train_gt_unique.txt',
                            'TRAIN_IMGS_LIST': '%s/annotations/train_imgs.txt',
                            'TEST_GT_LABELS': '%s/annotations/test_gt_labels.txt',
                            'TEST_GT_UNIQUE': '%s/annotations/test_gt_unique.txt',
                            'TEST_IMGS_LIST': '%s/annotations/test_imgs.txt',
                            'INPUTS_IDS_DATASET': ['menu', 'features', 'yolo_predictions', 'correlations'],
                            'OUTPUTS_IDS_DATASET': ['output', 'output2'],
                            'INPUTS': {
                                'menu': {
                                    'type': 'text', # 'raw-image' for single images and 'video' for group images
                                    'words_as_classes': True,  # specific type text param
                                    'max_text_len': 35, # specific type text param
                                    'path': 'annotations/%s_menu.txt',
                                    'num_classes': 104,
                                },
                                'features': {
                                    'type': 'video-features', # 'raw-image' for single images and 'video' for group images
                                    'path': 'annotations/%s_features.txt',
                                    'frames_path': 'annotations/%s_frames.txt', # only for type video,
                                    'indices_path': 'annotations/%s_video_indices.txt', # only for type video,
                                    'max_video_len': 20,
                                    'feat_len': 1536,
                                    'activation': 'tanh',
                                    'normalization': ['L2'],  # For video-features only accepted L2 or None
                                    'data_augmentation': ['noise'],  # For video-features only accepted noise, random_selections or None
                                },
                                'yolo_predictions': {
                                    'type': 'text', # 'raw-image' for single images and 'video' for group images
                                    'words_as_classes': True,  # specific type text param
                                    'max_text_len': 20, # specific type text param
                                    'path': 'annotations/%s_pred_labels.txt',
                                },
                                'correlations': {
                                    'type': 'text', # 'raw-image' for single images and 'video' for group images
                                    'words_as_classes': True,  # specific type text param
                                    'max_text_len': 20, # specific type text param
                                    'path': 'annotations/%s_pred_labels.txt',
                                    'num_classes': 88,
                                },
                            },
                            'OUTPUTS': {
                                'output': {
                                    'type': 'text',   # 'categorical' for single images and 'text' for group images
                                    'words_as_classes': True,  # specific type text param
                                    'max_text_len': 20, # specific type text param
                                    'path': 'annotations/%s_gt_labels.txt',
                                    'classes': 'annotations/yolo_classes_new.txt',
                                    'metrics': ['multiclass_metrics_custom'],  # multilabel_metrics or multiclass_metrics
                                    'write_type': 'listoflists',  # listoflists or list
                                    'activation': 'softmax',  # sigmoid or softmax
                                    'loss': 'categorical_crossentropy',  # binary_crossentropy or categorical_crossentropy
                                    'save_classes_report': False,  # whether or not to save the classification report for each class
                                },
                                'output2': {
                                    'type': 'text',   # 'categorical' for single images and 'text' for group images
                                    'words_as_classes': True,  # specific type text param
                                    'max_text_len': 20, # specific type text param
                                    'path': 'annotations/%s_gt_labels.txt',
                                    'classes': 'annotations/yolo_classes_new.txt',
                                    'metrics': ['multiclass_metrics_custom'],  # multilabel_metrics or multiclass_metrics
                                    'write_type': 'listoflists',  # listoflists or list
                                    'activation': 'softmax',  # sigmoid or softmax
                                    'loss': 'categorical_crossentropy',  # binary_crossentropy or categorical_crossentropy
                                    'save_classes_report': False,  # whether or not to save the classification report for each class
                                },
                            },
                            # Training params
                            'SPLITS': ['test', 'train'], # ['train','val','test']
                            'EMPTY_LABEL': False,
                            'RELOAD': 0,  #30, #39
                            'PATIENCE': 5,
                            'STOP_METRIC': 'accuracy',
                            'BATCH_SIZE': 24,
                            'LR_DECAY': None,  # number of minimum number of epochs before the next LR decay (set to None to disable)
                            'LR_GAMMA': 0.5,  # multiplier used for decreasing the LR
                            'SHUFFLE': True,
                            'MAX_FRAME': 20,
                            'EMBEDDING_SIZE': 50,
                            'FUSION_INPUT_LEN': 200,
                            'DROPOUT': 0.2, 
                            'WEIGHT_DECAY': 1e-3,  # L2 regularization
                            'RECURRENT_WEIGHT_DECAY': 0.0,  # L2 regularization in recurrent layers
                            'USE_RECURRENT_DROPOUT': False,  # Use dropout in recurrent layers # DANGEROUS!
                            'RECURRENT_DROPOUT_P': 0.5,  # Percentage of units to drop in recurrent layers
                            'INIT_LAYERS': ['tanh'],
                            'DECODER_HIDDEN_SIZE': 484,  # For models with LSTM decoder (ABiViRNet 484)
                            'TARGET_TEXT_EMBEDDING_SIZE': 99,
                            'ADDITIONAL_OUTPUT_MERGE_MODE': 'sum',
                            'CLASSIFIER_ACTIVATION': 'softmax',
                            'EPSILON': 1e-7,
                            'VOCABULARY_SIZE': 104,
                            'MIN_OCCURRENCES_VOCAB': 0,

                            # Prediction params
                            'SAMPLING_SAVE_MODE': 'numpy',
                            'SAMPLING_RELOAD_POINT': 30,
                            'SAMPLING_RELOAD_EPOCH': True,
                            'TOKENIZATION_METHOD': None,
                            'SAMPLING': 'max_likelihood',  # Possible values: multinomial or max_likelihood (recommended)
                            'TEMPERATURE': 1,
                        },

                        ###################
                        # Model fusion: (FV+YP) + (C+M)
                        ###################
                        'FV_YP_fusion_C_M_model': {
                            # Model params
                            'MODEL_TYPE': 'FV_YP_fusion_C_M_model',  # see predefined models in models.py
                            'MODEL_NAME': '(FV+YP) + (C+M) MODEL', # 'TFG_VIDEO'
                            'STORE_PATH': '%s/FV_YP_fusion_C_M_model',  # insert MODELS_ROOT_PATH in %s. '%s/tfg_video'
                            'MODELS_ROOT_PATH': '/models',
                            'EVALUATE': True,
                            # Dataset params
                            'DATA_ROOT_PATH': '/data_yolo',  # '/data' for single images and '/data_video' for group images
                            'DATASET_NAME': 'Food',
                            'REBUILD_DATASET': True,
                            'ALL_CLASSES_NAMES_PATH': '%s/annotations/yolo_classes_new.txt',
                            'YOLO_NAMES': '%s/annotations/yolo.names',
                            'TRAIN_GT_LABELS': '%s/annotations/train_gt_labels.txt',
                            'TRAIN_GT_UNIQUE': '%s/annotations/train_gt_unique.txt',
                            'TRAIN_IMGS_LIST': '%s/annotations/train_imgs.txt',
                            'TEST_GT_LABELS': '%s/annotations/test_gt_labels.txt',
                            'TEST_GT_UNIQUE': '%s/annotations/test_gt_unique.txt',
                            'TEST_IMGS_LIST': '%s/annotations/test_imgs.txt',
                            'INPUTS_IDS_DATASET': ['menu', 'features', 'yolo_predictions', 'correlations'],
                            'OUTPUTS_IDS_DATASET': ['output', 'output2'],
                            'INPUTS': {
                                'menu': {
                                    'type': 'text', # 'raw-image' for single images and 'video' for group images
                                    'words_as_classes': True,  # specific type text param
                                    'max_text_len': 35, # specific type text param
                                    'path': 'annotations/%s_menu.txt',
                                    'num_classes': 104,
                                },
                                'features': {
                                    'type': 'video-features', # 'raw-image' for single images and 'video' for group images
                                    'path': 'annotations/%s_features.txt',
                                    'frames_path': 'annotations/%s_frames.txt', # only for type video,
                                    'indices_path': 'annotations/%s_video_indices.txt', # only for type video,
                                    'max_video_len': 20,
                                    'feat_len': 1536,
                                    'activation': 'tanh',
                                    'normalization': ['L2'],  # For video-features only accepted L2 or None
                                    'data_augmentation': ['noise'],  # For video-features only accepted noise, random_selections or None
                                },
                                'yolo_predictions': {
                                    'type': 'text', # 'raw-image' for single images and 'video' for group images
                                    'words_as_classes': True,  # specific type text param
                                    'max_text_len': 20, # specific type text param
                                    'path': 'annotations/%s_pred_labels.txt',
                                },
                                'correlations': {
                                    'type': 'text', # 'raw-image' for single images and 'video' for group images
                                    'words_as_classes': True,  # specific type text param
                                    'max_text_len': 20, # specific type text param
                                    'path': 'annotations/%s_pred_labels.txt',
                                    'num_classes': 88,
                                },
                            },
                            'OUTPUTS': {
                                'output': {
                                    'type': 'text',   # 'categorical' for single images and 'text' for group images
                                    'words_as_classes': True,  # specific type text param
                                    'max_text_len': 20, # specific type text param
                                    'path': 'annotations/%s_gt_labels.txt',
                                    'classes': 'annotations/yolo_classes_new.txt',
                                    'metrics': ['multiclass_metrics_custom'],  # multilabel_metrics or multiclass_metrics
                                    'write_type': 'listoflists',  # listoflists or list
                                    'activation': 'softmax',  # sigmoid or softmax
                                    'loss': 'categorical_crossentropy',  # binary_crossentropy or categorical_crossentropy
                                    'save_classes_report': False,  # whether or not to save the classification report for each class
                                },
                                'output2': {
                                    'type': 'text',   # 'categorical' for single images and 'text' for group images
                                    'words_as_classes': True,  # specific type text param
                                    'max_text_len': 20, # specific type text param
                                    'path': 'annotations/%s_gt_labels.txt',
                                    'classes': 'annotations/yolo_classes_new.txt',
                                    'metrics': ['multiclass_metrics_custom'],  # multilabel_metrics or multiclass_metrics
                                    'write_type': 'listoflists',  # listoflists or list
                                    'activation': 'softmax',  # sigmoid or softmax
                                    'loss': 'categorical_crossentropy',  # binary_crossentropy or categorical_crossentropy
                                    'save_classes_report': False,  # whether or not to save the classification report for each class
                                },
                            },
                            # Training params
                            'SPLITS': ['test', 'train'], # ['train','val','test']
                            'EMPTY_LABEL': False,
                            'RELOAD': 0,  #30, #39
                            'PATIENCE': 5,
                            'STOP_METRIC': 'accuracy',
                            'BATCH_SIZE': 24,
                            'LR_DECAY': None,  # number of minimum number of epochs before the next LR decay (set to None to disable)
                            'LR_GAMMA': 0.5,  # multiplier used for decreasing the LR
                            'SHUFFLE': True,
                            'MAX_FRAME': 20,
                            'EMBEDDING_SIZE': 50,
                            'FUSION_INPUT_LEN': 200,
                            'DROPOUT': 0.2, 
                            'WEIGHT_DECAY': 1e-3,  # L2 regularization
                            'RECURRENT_WEIGHT_DECAY': 0.0,  # L2 regularization in recurrent layers
                            'USE_RECURRENT_DROPOUT': False,  # Use dropout in recurrent layers # DANGEROUS!
                            'RECURRENT_DROPOUT_P': 0.5,  # Percentage of units to drop in recurrent layers
                            'INIT_LAYERS': ['tanh'],
                            'DECODER_HIDDEN_SIZE': 484,  # For models with LSTM decoder (ABiViRNet 484)
                            'TARGET_TEXT_EMBEDDING_SIZE': 99,
                            'ADDITIONAL_OUTPUT_MERGE_MODE': 'sum',
                            'CLASSIFIER_ACTIVATION': 'softmax',
                            'EPSILON': 1e-7,
                            'VOCABULARY_SIZE': 104,
                            'MIN_OCCURRENCES_VOCAB': 0,

                            # Prediction params
                            'SAMPLING_SAVE_MODE': 'numpy',
                            'SAMPLING_RELOAD_POINT': 30,
                            'SAMPLING_RELOAD_EPOCH': True,
                            'TOKENIZATION_METHOD': None,
                            'SAMPLING': 'max_likelihood',  # Possible values: multinomial or max_likelihood (recommended)
                            'TEMPERATURE': 1,
                        },

                        ###################
                        # Model fusion: (FV+YP) * (C+M)
                        ###################
                        'FV_YP_mult_C_M_model': {
                            # Model params
                            'MODEL_TYPE': 'FV_YP_mult_C_M_model',  # see predefined models in models.py
                            'MODEL_NAME': '(FV+YP) * (C+M) MODEL', # 'TFG_VIDEO'
                            'STORE_PATH': '%s/FV_YP_mult_C_M_model',  # insert MODELS_ROOT_PATH in %s. '%s/tfg_video'
                            'MODELS_ROOT_PATH': '/models',
                            'EVALUATE': True,
                            # Dataset params
                            'DATA_ROOT_PATH': '/data_yolo',  # '/data' for single images and '/data_video' for group images
                            'DATASET_NAME': 'Food',
                            'REBUILD_DATASET': True,
                            'ALL_CLASSES_NAMES_PATH': '%s/annotations/yolo_classes_new.txt',
                            'YOLO_NAMES': '%s/annotations/yolo.names',
                            'TRAIN_GT_LABELS': '%s/annotations/train_gt_labels.txt',
                            'TRAIN_GT_UNIQUE': '%s/annotations/train_gt_unique.txt',
                            'TRAIN_IMGS_LIST': '%s/annotations/train_imgs.txt',
                            'TEST_GT_LABELS': '%s/annotations/test_gt_labels.txt',
                            'TEST_GT_UNIQUE': '%s/annotations/test_gt_unique.txt',
                            'TEST_IMGS_LIST': '%s/annotations/test_imgs.txt',
                            'INPUTS_IDS_DATASET': ['menu', 'features', 'yolo_predictions', 'correlations'],
                            'OUTPUTS_IDS_DATASET': ['output', 'output2'],
                            'INPUTS': {
                                'menu': {
                                    'type': 'text', # 'raw-image' for single images and 'video' for group images
                                    'words_as_classes': True,  # specific type text param
                                    'max_text_len': 35, # specific type text param
                                    'path': 'annotations/%s_menu.txt',
                                    'num_classes': 104,
                                },
                                'features': {
                                    'type': 'video-features', # 'raw-image' for single images and 'video' for group images
                                    'path': 'annotations/%s_features.txt',
                                    'frames_path': 'annotations/%s_frames.txt', # only for type video,
                                    'indices_path': 'annotations/%s_video_indices.txt', # only for type video,
                                    'max_video_len': 20,
                                    'feat_len': 1536,
                                    'activation': 'tanh',
                                    'normalization': ['L2'],  # For video-features only accepted L2 or None
                                    'data_augmentation': ['noise'],  # For video-features only accepted noise, random_selections or None
                                },
                                'yolo_predictions': {
                                    'type': 'text', # 'raw-image' for single images and 'video' for group images
                                    'words_as_classes': True,  # specific type text param
                                    'max_text_len': 20, # specific type text param
                                    'path': 'annotations/%s_pred_labels.txt',
                                },
                                'correlations': {
                                    'type': 'text', # 'raw-image' for single images and 'video' for group images
                                    'words_as_classes': True,  # specific type text param
                                    'max_text_len': 20, # specific type text param
                                    'path': 'annotations/%s_pred_labels.txt',
                                    'num_classes': 88,
                                },
                            },
                            'OUTPUTS': {
                                'output': {
                                    'type': 'text',   # 'categorical' for single images and 'text' for group images
                                    'words_as_classes': True,  # specific type text param
                                    'max_text_len': 20, # specific type text param
                                    'path': 'annotations/%s_gt_labels.txt',
                                    'classes': 'annotations/yolo_classes_new.txt',
                                    'metrics': ['multiclass_metrics_custom'],  # multilabel_metrics or multiclass_metrics
                                    'write_type': 'listoflists',  # listoflists or list
                                    'activation': 'softmax',  # sigmoid or softmax
                                    'loss': 'categorical_crossentropy',  # binary_crossentropy or categorical_crossentropy
                                    'save_classes_report': False,  # whether or not to save the classification report for each class
                                },
                                'output2': {
                                    'type': 'text',   # 'categorical' for single images and 'text' for group images
                                    'words_as_classes': True,  # specific type text param
                                    'max_text_len': 20, # specific type text param
                                    'path': 'annotations/%s_gt_labels.txt',
                                    'classes': 'annotations/yolo_classes_new.txt',
                                    'metrics': ['multiclass_metrics_custom'],  # multilabel_metrics or multiclass_metrics
                                    'write_type': 'listoflists',  # listoflists or list
                                    'activation': 'softmax',  # sigmoid or softmax
                                    'loss': 'categorical_crossentropy',  # binary_crossentropy or categorical_crossentropy
                                    'save_classes_report': False,  # whether or not to save the classification report for each class
                                },
                            },
                            # Training params
                            'SPLITS': ['test', 'train'], # ['train','val','test']
                            'EMPTY_LABEL': False,
                            'RELOAD': 0,  #30, #39
                            'PATIENCE': 5,
                            'STOP_METRIC': 'accuracy',
                            'BATCH_SIZE': 24,
                            'LR_DECAY': None,  # number of minimum number of epochs before the next LR decay (set to None to disable)
                            'LR_GAMMA': 0.5,  # multiplier used for decreasing the LR
                            'SHUFFLE': True,
                            'MAX_FRAME': 20,
                            'EMBEDDING_SIZE': 50,
                            'FUSION_INPUT_LEN': 200,
                            'DROPOUT': 0.2, 
                            'WEIGHT_DECAY': 1e-3,  # L2 regularization
                            'RECURRENT_WEIGHT_DECAY': 0.0,  # L2 regularization in recurrent layers
                            'USE_RECURRENT_DROPOUT': False,  # Use dropout in recurrent layers # DANGEROUS!
                            'RECURRENT_DROPOUT_P': 0.5,  # Percentage of units to drop in recurrent layers
                            'INIT_LAYERS': ['tanh'],
                            'DECODER_HIDDEN_SIZE': 484,  # For models with LSTM decoder (ABiViRNet 484)
                            'TARGET_TEXT_EMBEDDING_SIZE': 99,
                            'ADDITIONAL_OUTPUT_MERGE_MODE': 'sum',
                            'CLASSIFIER_ACTIVATION': 'softmax',
                            'EPSILON': 1e-7,
                            'VOCABULARY_SIZE': 104,
                            'MIN_OCCURRENCES_VOCAB': 0,

                            # Prediction params
                            'SAMPLING_SAVE_MODE': 'numpy',
                            'SAMPLING_RELOAD_POINT': 30,
                            'SAMPLING_RELOAD_EPOCH': True,
                            'TOKENIZATION_METHOD': None,
                            'SAMPLING': 'max_likelihood',  # Possible values: multinomial or max_likelihood (recommended)
                            'TEMPERATURE': 1,
                        },

                        ###################
                        # Model fusion: (FV+YP) + (C*M)
                        ###################
                        'FV_YP_fusion_C_M_mult_model': {
                            # Model params
                            'MODEL_TYPE': 'FV_YP_fusion_C_M_mult_model',  # see predefined models in models.py
                            'MODEL_NAME': '(FV+YP) + (C*M) MODEL', # 'TFG_VIDEO'
                            'STORE_PATH': '%s/FV_YP_fusion_C_M_mult_model',  # insert MODELS_ROOT_PATH in %s. '%s/tfg_video'
                            'MODELS_ROOT_PATH': '/models',
                            'EVALUATE': True,
                            # Dataset params
                            'DATA_ROOT_PATH': '/data_yolo',  # '/data' for single images and '/data_video' for group images
                            'DATASET_NAME': 'Food',
                            'REBUILD_DATASET': True,
                            'ALL_CLASSES_NAMES_PATH': '%s/annotations/yolo_classes_new.txt',
                            'YOLO_NAMES': '%s/annotations/yolo.names',
                            'TRAIN_GT_LABELS': '%s/annotations/train_gt_labels.txt',
                            'TRAIN_GT_UNIQUE': '%s/annotations/train_gt_unique.txt',
                            'TRAIN_IMGS_LIST': '%s/annotations/train_imgs.txt',
                            'TEST_GT_LABELS': '%s/annotations/test_gt_labels.txt',
                            'TEST_GT_UNIQUE': '%s/annotations/test_gt_unique.txt',
                            'TEST_IMGS_LIST': '%s/annotations/test_imgs.txt',
                            'INPUTS_IDS_DATASET': ['menu', 'features', 'yolo_predictions', 'correlations'],
                            'OUTPUTS_IDS_DATASET': ['output', 'output2'],
                            'INPUTS': {
                                'menu': {
                                    'type': 'text', # 'raw-image' for single images and 'video' for group images
                                    'words_as_classes': True,  # specific type text param
                                    'max_text_len': 35, # specific type text param
                                    'path': 'annotations/%s_menu.txt',
                                    'num_classes': 104,
                                },
                                'features': {
                                    'type': 'video-features', # 'raw-image' for single images and 'video' for group images
                                    'path': 'annotations/%s_features.txt',
                                    'frames_path': 'annotations/%s_frames.txt', # only for type video,
                                    'indices_path': 'annotations/%s_video_indices.txt', # only for type video,
                                    'max_video_len': 20,
                                    'feat_len': 1536,
                                    'activation': 'tanh',
                                    'normalization': ['L2'],  # For video-features only accepted L2 or None
                                    'data_augmentation': ['noise'],  # For video-features only accepted noise, random_selections or None
                                },
                                'yolo_predictions': {
                                    'type': 'text', # 'raw-image' for single images and 'video' for group images
                                    'words_as_classes': True,  # specific type text param
                                    'max_text_len': 20, # specific type text param
                                    'path': 'annotations/%s_pred_labels.txt',
                                },
                                'correlations': {
                                    'type': 'text', # 'raw-image' for single images and 'video' for group images
                                    'words_as_classes': True,  # specific type text param
                                    'max_text_len': 20, # specific type text param
                                    'path': 'annotations/%s_pred_labels.txt',
                                    'num_classes': 88,
                                },
                            },
                            'OUTPUTS': {
                                'output': {
                                    'type': 'text',   # 'categorical' for single images and 'text' for group images
                                    'words_as_classes': True,  # specific type text param
                                    'max_text_len': 20, # specific type text param
                                    'path': 'annotations/%s_gt_labels.txt',
                                    'classes': 'annotations/yolo_classes_new.txt',
                                    'metrics': ['multiclass_metrics_custom'],  # multilabel_metrics or multiclass_metrics
                                    'write_type': 'listoflists',  # listoflists or list
                                    'activation': 'softmax',  # sigmoid or softmax
                                    'loss': 'categorical_crossentropy',  # binary_crossentropy or categorical_crossentropy
                                    'save_classes_report': False,  # whether or not to save the classification report for each class
                                },
                                'output2': {
                                    'type': 'text',   # 'categorical' for single images and 'text' for group images
                                    'words_as_classes': True,  # specific type text param
                                    'max_text_len': 20, # specific type text param
                                    'path': 'annotations/%s_gt_labels.txt',
                                    'classes': 'annotations/yolo_classes_new.txt',
                                    'metrics': ['multiclass_metrics_custom'],  # multilabel_metrics or multiclass_metrics
                                    'write_type': 'listoflists',  # listoflists or list
                                    'activation': 'softmax',  # sigmoid or softmax
                                    'loss': 'categorical_crossentropy',  # binary_crossentropy or categorical_crossentropy
                                    'save_classes_report': False,  # whether or not to save the classification report for each class
                                },
                            },
                            # Training params
                            'SPLITS': ['test', 'train'], # ['train','val','test']
                            'EMPTY_LABEL': False,
                            'RELOAD': 0,  #30, #39
                            'PATIENCE': 5,
                            'STOP_METRIC': 'accuracy',
                            'BATCH_SIZE': 24,
                            'LR_DECAY': None,  # number of minimum number of epochs before the next LR decay (set to None to disable)
                            'LR_GAMMA': 0.5,  # multiplier used for decreasing the LR
                            'SHUFFLE': True,
                            'MAX_FRAME': 20,
                            'EMBEDDING_SIZE': 50,
                            'FUSION_INPUT_LEN': 200,
                            'DROPOUT': 0.2, 
                            'WEIGHT_DECAY': 1e-3,  # L2 regularization
                            'RECURRENT_WEIGHT_DECAY': 0.0,  # L2 regularization in recurrent layers
                            'USE_RECURRENT_DROPOUT': False,  # Use dropout in recurrent layers # DANGEROUS!
                            'RECURRENT_DROPOUT_P': 0.5,  # Percentage of units to drop in recurrent layers
                            'INIT_LAYERS': ['tanh'],
                            'DECODER_HIDDEN_SIZE': 484,  # For models with LSTM decoder (ABiViRNet 484)
                            'TARGET_TEXT_EMBEDDING_SIZE': 99,
                            'ADDITIONAL_OUTPUT_MERGE_MODE': 'mul',
                            'CLASSIFIER_ACTIVATION': 'softmax',
                            'EPSILON': 1e-7,
                            'VOCABULARY_SIZE': 104,
                            'MIN_OCCURRENCES_VOCAB': 0,

                            # Prediction params
                            'SAMPLING_SAVE_MODE': 'numpy',
                            'SAMPLING_RELOAD_POINT': 30,
                            'SAMPLING_RELOAD_EPOCH': True,
                            'TOKENIZATION_METHOD': None,
                            'SAMPLING': 'max_likelihood',  # Possible values: multinomial or max_likelihood (recommended)
                            'TEMPERATURE': 1,
                        },

                        ###################
                        # Model fusion: (FV+YP) * (C*M)
                        ###################
                        'FV_YP_mult_C_M_mult_model': {
                            # Model params
                            'MODEL_TYPE': 'FV_YP_mult_C_M_mult_model',  # see predefined models in models.py
                            'MODEL_NAME': '(FV+YP) * (C*M) MODEL', # 'TFG_VIDEO'
                            'STORE_PATH': '%s/FV_YP_mult_C_M_mult_model',  # insert MODELS_ROOT_PATH in %s. '%s/tfg_video'
                            'MODELS_ROOT_PATH': '/models',
                            'EVALUATE': True,
                            # Dataset params
                            'DATA_ROOT_PATH': '/data_yolo',  # '/data' for single images and '/data_video' for group images
                            'DATASET_NAME': 'Food',
                            'REBUILD_DATASET': True,
                            'ALL_CLASSES_NAMES_PATH': '%s/annotations/yolo_classes_new.txt',
                            'YOLO_NAMES': '%s/annotations/yolo.names',
                            'TRAIN_GT_LABELS': '%s/annotations/train_gt_labels.txt',
                            'TRAIN_GT_UNIQUE': '%s/annotations/train_gt_unique.txt',
                            'TRAIN_IMGS_LIST': '%s/annotations/train_imgs.txt',
                            'TEST_GT_LABELS': '%s/annotations/test_gt_labels.txt',
                            'TEST_GT_UNIQUE': '%s/annotations/test_gt_unique.txt',
                            'TEST_IMGS_LIST': '%s/annotations/test_imgs.txt',
                            'INPUTS_IDS_DATASET': ['menu', 'features', 'yolo_predictions', 'correlations'],
                            'OUTPUTS_IDS_DATASET': ['output', 'output2'],
                            'INPUTS': {
                                'menu': {
                                    'type': 'text', # 'raw-image' for single images and 'video' for group images
                                    'words_as_classes': True,  # specific type text param
                                    'max_text_len': 35, # specific type text param
                                    'path': 'annotations/%s_menu.txt',
                                    'num_classes': 104,
                                },
                                'features': {
                                    'type': 'video-features', # 'raw-image' for single images and 'video' for group images
                                    'path': 'annotations/%s_features.txt',
                                    'frames_path': 'annotations/%s_frames.txt', # only for type video,
                                    'indices_path': 'annotations/%s_video_indices.txt', # only for type video,
                                    'max_video_len': 20,
                                    'feat_len': 1536,
                                    'activation': 'tanh',
                                    'normalization': ['L2'],  # For video-features only accepted L2 or None
                                    'data_augmentation': ['noise'],  # For video-features only accepted noise, random_selections or None
                                },
                                'yolo_predictions': {
                                    'type': 'text', # 'raw-image' for single images and 'video' for group images
                                    'words_as_classes': True,  # specific type text param
                                    'max_text_len': 20, # specific type text param
                                    'path': 'annotations/%s_pred_labels.txt',
                                },
                                'correlations': {
                                    'type': 'text', # 'raw-image' for single images and 'video' for group images
                                    'words_as_classes': True,  # specific type text param
                                    'max_text_len': 20, # specific type text param
                                    'path': 'annotations/%s_pred_labels.txt',
                                    'num_classes': 88,
                                },
                            },
                            'OUTPUTS': {
                                'output': {
                                    'type': 'text',   # 'categorical' for single images and 'text' for group images
                                    'words_as_classes': True,  # specific type text param
                                    'max_text_len': 20, # specific type text param
                                    'path': 'annotations/%s_gt_labels.txt',
                                    'classes': 'annotations/yolo_classes_new.txt',
                                    'metrics': ['multiclass_metrics_custom'],  # multilabel_metrics or multiclass_metrics
                                    'write_type': 'listoflists',  # listoflists or list
                                    'activation': 'softmax',  # sigmoid or softmax
                                    'loss': 'categorical_crossentropy',  # binary_crossentropy or categorical_crossentropy
                                    'save_classes_report': False,  # whether or not to save the classification report for each class
                                },
                                'output2': {
                                    'type': 'text',   # 'categorical' for single images and 'text' for group images
                                    'words_as_classes': True,  # specific type text param
                                    'max_text_len': 20, # specific type text param
                                    'path': 'annotations/%s_gt_labels.txt',
                                    'classes': 'annotations/yolo_classes_new.txt',
                                    'metrics': ['multiclass_metrics_custom'],  # multilabel_metrics or multiclass_metrics
                                    'write_type': 'listoflists',  # listoflists or list
                                    'activation': 'softmax',  # sigmoid or softmax
                                    'loss': 'categorical_crossentropy',  # binary_crossentropy or categorical_crossentropy
                                    'save_classes_report': False,  # whether or not to save the classification report for each class
                                },
                            },
                            # Training params
                            'SPLITS': ['test', 'train'], # ['train','val','test']
                            'EMPTY_LABEL': False,
                            'RELOAD': 0,  #30, #39
                            'PATIENCE': 5,
                            'STOP_METRIC': 'accuracy',
                            'BATCH_SIZE': 24,
                            'LR_DECAY': None,  # number of minimum number of epochs before the next LR decay (set to None to disable)
                            'LR_GAMMA': 0.5,  # multiplier used for decreasing the LR
                            'SHUFFLE': True,
                            'MAX_FRAME': 20,
                            'EMBEDDING_SIZE': 50,
                            'FUSION_INPUT_LEN': 200,
                            'DROPOUT': 0.2, 
                            'WEIGHT_DECAY': 1e-3,  # L2 regularization
                            'RECURRENT_WEIGHT_DECAY': 0.0,  # L2 regularization in recurrent layers
                            'USE_RECURRENT_DROPOUT': False,  # Use dropout in recurrent layers # DANGEROUS!
                            'RECURRENT_DROPOUT_P': 0.5,  # Percentage of units to drop in recurrent layers
                            'INIT_LAYERS': ['tanh'],
                            'DECODER_HIDDEN_SIZE': 484,  # For models with LSTM decoder (ABiViRNet 484)
                            'TARGET_TEXT_EMBEDDING_SIZE': 99,
                            'ADDITIONAL_OUTPUT_MERGE_MODE': 'sum',
                            'CLASSIFIER_ACTIVATION': 'softmax',
                            'EPSILON': 1e-7,
                            'VOCABULARY_SIZE': 104,
                            'MIN_OCCURRENCES_VOCAB': 0,

                            # Prediction params
                            'SAMPLING_SAVE_MODE': 'numpy',
                            'SAMPLING_RELOAD_POINT': 30,
                            'SAMPLING_RELOAD_EPOCH': True,
                            'TOKENIZATION_METHOD': None,
                            'SAMPLING': 'max_likelihood',  # Possible values: multinomial or max_likelihood (recommended)
                            'TEMPERATURE': 1,
                        },

                        ###################
                        # Model fusion: (FV+YP+C) + M
                        ###################
                        'FV_YP_C_fusion_M_model': {
                            # Model params
                            'MODEL_TYPE': 'FV_YP_C_fusion_M_model',  # see predefined models in models.py
                            'MODEL_NAME': '(FV+YP+C) + M MODEL', # 'TFG_VIDEO'
                            'STORE_PATH': '%s/FV_YP_C_fusion_M_model',  # insert MODELS_ROOT_PATH in %s. '%s/tfg_video'
                            'MODELS_ROOT_PATH': '/models',
                            'EVALUATE': True,
                            # Dataset params
                            'DATA_ROOT_PATH': '/data_yolo',  # '/data' for single images and '/data_video' for group images
                            'DATASET_NAME': 'Food',
                            'REBUILD_DATASET': True,
                            'ALL_CLASSES_NAMES_PATH': '%s/annotations/yolo_classes_new.txt',
                            'YOLO_NAMES': '%s/annotations/yolo.names',
                            'TRAIN_GT_LABELS': '%s/annotations/train_gt_labels.txt',
                            'TRAIN_GT_UNIQUE': '%s/annotations/train_gt_unique.txt',
                            'TRAIN_IMGS_LIST': '%s/annotations/train_imgs.txt',
                            'TEST_GT_LABELS': '%s/annotations/test_gt_labels.txt',
                            'TEST_GT_UNIQUE': '%s/annotations/test_gt_unique.txt',
                            'TEST_IMGS_LIST': '%s/annotations/test_imgs.txt',
                            'INPUTS_IDS_DATASET': ['menu', 'features', 'yolo_predictions', 'correlations'],
                            'OUTPUTS_IDS_DATASET': ['output', 'output2'],
                            'INPUTS': {
                                'menu': {
                                    'type': 'text', # 'raw-image' for single images and 'video' for group images
                                    'words_as_classes': True,  # specific type text param
                                    'max_text_len': 35, # specific type text param
                                    'path': 'annotations/%s_menu.txt',
                                    'num_classes': 104,
                                },
                                'features': {
                                    'type': 'video-features', # 'raw-image' for single images and 'video' for group images
                                    'path': 'annotations/%s_features.txt',
                                    'frames_path': 'annotations/%s_frames.txt', # only for type video,
                                    'indices_path': 'annotations/%s_video_indices.txt', # only for type video,
                                    'max_video_len': 20,
                                    'feat_len': 1536,
                                    'activation': 'tanh',
                                    'normalization': ['L2'],  # For video-features only accepted L2 or None
                                    'data_augmentation': ['noise'],  # For video-features only accepted noise, random_selections or None
                                },
                                'yolo_predictions': {
                                    'type': 'text', # 'raw-image' for single images and 'video' for group images
                                    'words_as_classes': True,  # specific type text param
                                    'max_text_len': 20, # specific type text param
                                    'path': 'annotations/%s_pred_labels.txt',
                                },
                                'correlations': {
                                    'type': 'text', # 'raw-image' for single images and 'video' for group images
                                    'words_as_classes': True,  # specific type text param
                                    'max_text_len': 20, # specific type text param
                                    'path': 'annotations/%s_pred_labels.txt',
                                    'num_classes': 88,
                                },
                            },
                            'OUTPUTS': {
                                'output': {
                                    'type': 'text',   # 'categorical' for single images and 'text' for group images
                                    'words_as_classes': True,  # specific type text param
                                    'max_text_len': 20, # specific type text param
                                    'path': 'annotations/%s_gt_labels.txt',
                                    'classes': 'annotations/yolo_classes_new.txt',
                                    'metrics': ['multiclass_metrics_custom'],  # multilabel_metrics or multiclass_metrics
                                    'write_type': 'listoflists',  # listoflists or list
                                    'activation': 'softmax',  # sigmoid or softmax
                                    'loss': 'categorical_crossentropy',  # binary_crossentropy or categorical_crossentropy
                                    'save_classes_report': False,  # whether or not to save the classification report for each class
                                },
                                'output2': {
                                    'type': 'text',   # 'categorical' for single images and 'text' for group images
                                    'words_as_classes': True,  # specific type text param
                                    'max_text_len': 20, # specific type text param
                                    'path': 'annotations/%s_gt_labels.txt',
                                    'classes': 'annotations/yolo_classes_new.txt',
                                    'metrics': ['multiclass_metrics_custom'],  # multilabel_metrics or multiclass_metrics
                                    'write_type': 'listoflists',  # listoflists or list
                                    'activation': 'softmax',  # sigmoid or softmax
                                    'loss': 'categorical_crossentropy',  # binary_crossentropy or categorical_crossentropy
                                    'save_classes_report': False,  # whether or not to save the classification report for each class
                                },
                            },
                            # Training params
                            'SPLITS': ['test', 'train'], # ['train','val','test']
                            'EMPTY_LABEL': False,
                            'RELOAD': 0,  #30, #39
                            'PATIENCE': 5,
                            'STOP_METRIC': 'accuracy',
                            'BATCH_SIZE': 24,
                            'LR_DECAY': None,  # number of minimum number of epochs before the next LR decay (set to None to disable)
                            'LR_GAMMA': 0.5,  # multiplier used for decreasing the LR
                            'SHUFFLE': True,
                            'MAX_FRAME': 20,
                            'EMBEDDING_SIZE': 50,
                            'FUSION_INPUT_LEN': 200,
                            'DROPOUT': 0.2, 
                            'WEIGHT_DECAY': 1e-3,  # L2 regularization
                            'RECURRENT_WEIGHT_DECAY': 0.0,  # L2 regularization in recurrent layers
                            'USE_RECURRENT_DROPOUT': False,  # Use dropout in recurrent layers # DANGEROUS!
                            'RECURRENT_DROPOUT_P': 0.5,  # Percentage of units to drop in recurrent layers
                            'INIT_LAYERS': ['tanh'],
                            'DECODER_HIDDEN_SIZE': 484,  # For models with LSTM decoder (ABiViRNet 484)
                            'TARGET_TEXT_EMBEDDING_SIZE': 99,
                            'ADDITIONAL_OUTPUT_MERGE_MODE': 'sum',
                            'CLASSIFIER_ACTIVATION': 'softmax',
                            'EPSILON': 1e-7,
                            'VOCABULARY_SIZE': 104,
                            'MIN_OCCURRENCES_VOCAB': 0,

                            # Prediction params
                            'SAMPLING_SAVE_MODE': 'numpy',
                            'SAMPLING_RELOAD_POINT': 30,
                            'SAMPLING_RELOAD_EPOCH': True,
                            'TOKENIZATION_METHOD': None,
                            'SAMPLING': 'max_likelihood',  # Possible values: multinomial or max_likelihood (recommended)
                            'TEMPERATURE': 1,
                        },

                        ###################
                        # Model fusion: (FV + YP) + (FV + C) + (FV + M)   
                        ###################
                        'FV_YP_fusion_FV_C_fusion_FV_M_model': {
                            # Model params
                            'MODEL_TYPE': 'FV_YP_fusion_FV_C_fusion_FV_M_model',  # see predefined models in models.py
                            'MODEL_NAME': '(FV + YP) + (FV + C) + (FV + M) MODEL', # 'TFG_VIDEO'
                            'STORE_PATH': '%s/FV_YP_fusion_FV_C_fusion_FV_M_model',  # insert MODELS_ROOT_PATH in %s. '%s/tfg_video'
                            'MODELS_ROOT_PATH': '/models',
                            'EVALUATE': True,
                            # Dataset params
                            'DATA_ROOT_PATH': '/data_yolo',  # '/data' for single images and '/data_video' for group images
                            'DATASET_NAME': 'Food',
                            'REBUILD_DATASET': True,
                            'ALL_CLASSES_NAMES_PATH': '%s/annotations/yolo_classes_new.txt',
                            'YOLO_NAMES': '%s/annotations/yolo.names',
                            'TRAIN_GT_LABELS': '%s/annotations/train_gt_labels.txt',
                            'TRAIN_GT_UNIQUE': '%s/annotations/train_gt_unique.txt',
                            'TRAIN_IMGS_LIST': '%s/annotations/train_imgs.txt',
                            'TEST_GT_LABELS': '%s/annotations/test_gt_labels.txt',
                            'TEST_GT_UNIQUE': '%s/annotations/test_gt_unique.txt',
                            'TEST_IMGS_LIST': '%s/annotations/test_imgs.txt',
                            'INPUTS_IDS_DATASET': ['menu', 'features', 'yolo_predictions', 'correlations'],
                            'OUTPUTS_IDS_DATASET': ['output'],
                            'INPUTS': {
                                'menu': {
                                    'type': 'text', # 'raw-image' for single images and 'video' for group images
                                    'words_as_classes': True,  # specific type text param
                                    'max_text_len': 35, # specific type text param
                                    'path': 'annotations/%s_menu.txt',
                                    'num_classes': 104,
                                },
                                'features': {
                                    'type': 'video-features', # 'raw-image' for single images and 'video' for group images
                                    'path': 'annotations/%s_features.txt',
                                    'frames_path': 'annotations/%s_frames.txt', # only for type video,
                                    'indices_path': 'annotations/%s_video_indices.txt', # only for type video,
                                    'max_video_len': 20,
                                    'feat_len': 1536,
                                    'activation': 'tanh',
                                    'normalization': ['L2'],  # For video-features only accepted L2 or None
                                    'data_augmentation': ['noise'],  # For video-features only accepted noise, random_selections or None
                                },
                                'yolo_predictions': {
                                    'type': 'text', # 'raw-image' for single images and 'video' for group images
                                    'words_as_classes': True,  # specific type text param
                                    'max_text_len': 20, # specific type text param
                                    'path': 'annotations/%s_pred_labels.txt',
                                },
                                'correlations': {
                                    'type': 'text', # 'raw-image' for single images and 'video' for group images
                                    'words_as_classes': True,  # specific type text param
                                    'max_text_len': 20, # specific type text param
                                    'path': 'annotations/%s_pred_labels.txt',
                                    'num_classes': 88,
                                },
                            },
                            'OUTPUTS': {
                                'output': {
                                    'type': 'text',   # 'categorical' for single images and 'text' for group images
                                    'words_as_classes': True,  # specific type text param
                                    'max_text_len': 20, # specific type text param
                                    'path': 'annotations/%s_gt_labels.txt',
                                    'classes': 'annotations/yolo_classes_new.txt',
                                    'metrics': ['multiclass_metrics_custom'],  # multilabel_metrics or multiclass_metrics
                                    'write_type': 'listoflists',  # listoflists or list
                                    'activation': 'softmax',  # sigmoid or softmax
                                    'loss': 'categorical_crossentropy',  # binary_crossentropy or categorical_crossentropy
                                    'save_classes_report': False,  # whether or not to save the classification report for each class
                                }
                            },
                            # Training params
                            'SPLITS': ['test', 'train'], # ['train','val','test']
                            'EMPTY_LABEL': False,
                            'RELOAD': 0,  #30, #39
                            'PATIENCE': 5,
                            'STOP_METRIC': 'accuracy',
                            'BATCH_SIZE': 24,
                            'LR_DECAY': None,  # number of minimum number of epochs before the next LR decay (set to None to disable)
                            'LR_GAMMA': 0.5,  # multiplier used for decreasing the LR
                            'SHUFFLE': True,
                            'MAX_FRAME': 20,
                            'EMBEDDING_SIZE': 50,
                            'FUSION_INPUT_LEN': 200,
                            'DROPOUT': 0.5, 
                            'WEIGHT_DECAY': 1e-3,  # L2 regularization
                            'RECURRENT_WEIGHT_DECAY': 0.0,  # L2 regularization in recurrent layers
                            'USE_RECURRENT_DROPOUT': False,  # Use dropout in recurrent layers # DANGEROUS!
                            'RECURRENT_DROPOUT_P': 0.5,  # Percentage of units to drop in recurrent layers
                            'INIT_LAYERS': ['tanh'],
                            'DECODER_HIDDEN_SIZE': 484,  # For models with LSTM decoder (ABiViRNet 484)
                            'TARGET_TEXT_EMBEDDING_SIZE': 99,
                            'ADDITIONAL_OUTPUT_MERGE_MODE': 'sum',
                            'CLASSIFIER_ACTIVATION': 'softmax',
                            'EPSILON': 1e-7,
                            'VOCABULARY_SIZE': 104,
                            'MIN_OCCURRENCES_VOCAB': 0,

                            # Prediction params
                            'SAMPLING_SAVE_MODE': 'numpy',
                            'SAMPLING_RELOAD_POINT': 30,
                            'SAMPLING_RELOAD_EPOCH': True,
                            'TOKENIZATION_METHOD': None,
                            'SAMPLING': 'max_likelihood',  # Possible values: multinomial or max_likelihood (recommended)
                            'TEMPERATURE': 1,
                        },

                        ###################
                        # Model fusion: feature vectors + yolo predictions + correlations + menu
                        ###################
                        'FV_YP_C_M_model': {
                            # Model params
                            'MODEL_TYPE': 'FV_YP_C_M_model',  # see predefined models in models.py
                            'MODEL_NAME': 'FV+YP+C+M MODEL', # 'TFG_VIDEO'
                            'STORE_PATH': '%s/FV_YP_C_M_model',  # insert MODELS_ROOT_PATH in %s. '%s/tfg_video'
                            'MODELS_ROOT_PATH': '/models',
                            'EVALUATE': True,
                            # Dataset params
                            'DATA_ROOT_PATH': '/data_yolo',  # '/data' for single images and '/data_video' for group images
                            'DATASET_NAME': 'Food',
                            'REBUILD_DATASET': True,
                            'ALL_CLASSES_NAMES_PATH': '%s/annotations/yolo_classes_new.txt',
                            'YOLO_NAMES': '%s/annotations/yolo.names',
                            'TRAIN_GT_LABELS': '%s/annotations/train_gt_labels.txt',
                            'TRAIN_GT_UNIQUE': '%s/annotations/train_gt_unique.txt',
                            'TRAIN_IMGS_LIST': '%s/annotations/train_imgs.txt',
                            'TEST_GT_LABELS': '%s/annotations/test_gt_labels.txt',
                            'TEST_GT_UNIQUE': '%s/annotations/test_gt_unique.txt',
                            'TEST_IMGS_LIST': '%s/annotations/test_imgs.txt',
                            'INPUTS_IDS_DATASET': ['menu', 'features', 'yolo_predictions', 'correlations'],
                            'OUTPUTS_IDS_DATASET': ['output'],
                            'INPUTS': {
                                'menu': {
                                    'type': 'text', # 'raw-image' for single images and 'video' for group images
                                    'words_as_classes': True,  # specific type text param
                                    'max_text_len': 35, # specific type text param
                                    'path': 'annotations/%s_menu.txt',
                                    'num_classes': 104,
                                },
                                'features': {
                                    'type': 'video-features', # 'raw-image' for single images and 'video' for group images
                                    'path': 'annotations/%s_features.txt',
                                    'frames_path': 'annotations/%s_frames.txt', # only for type video,
                                    'indices_path': 'annotations/%s_video_indices.txt', # only for type video,
                                    'max_video_len': 20,
                                    'feat_len': 1536,
                                    'activation': 'tanh',
                                    'normalization': ['L2'],  # For video-features only accepted L2 or None
                                    'data_augmentation': ['noise'],  # For video-features only accepted noise, random_selections or None
                                },
                                'yolo_predictions': {
                                    'type': 'text', # 'raw-image' for single images and 'video' for group images
                                    'words_as_classes': True,  # specific type text param
                                    'max_text_len': 20, # specific type text param
                                    'path': 'annotations/%s_pred_labels.txt',
                                },
                                'correlations': {
                                    'type': 'text', # 'raw-image' for single images and 'video' for group images
                                    'words_as_classes': True,  # specific type text param
                                    'max_text_len': 20, # specific type text param
                                    'path': 'annotations/%s_pred_labels.txt',
                                    'num_classes': 88,
                                },
                            },
                            'OUTPUTS': {
                                'output': {
                                    'type': 'text',   # 'categorical' for single images and 'text' for group images
                                    'words_as_classes': True,  # specific type text param
                                    'max_text_len': 20, # specific type text param
                                    'path': 'annotations/%s_gt_labels.txt',
                                    'classes': 'annotations/yolo_classes_new.txt',
                                    'metrics': ['multiclass_metrics_custom'],  # multilabel_metrics or multiclass_metrics
                                    'write_type': 'listoflists',  # listoflists or list
                                    'activation': 'softmax',  # sigmoid or softmax
                                    'loss': 'categorical_crossentropy',  # binary_crossentropy or categorical_crossentropy
                                    'save_classes_report': False,  # whether or not to save the classification report for each class
                                }
                            },
                            # Training params
                            'SPLITS': ['test', 'train'], # ['train','val','test']
                            'EMPTY_LABEL': False,
                            'RELOAD': 0,  #30, #39
                            'PATIENCE': 5,
                            'STOP_METRIC': 'accuracy',
                            'BATCH_SIZE': 24,
                            'LR_DECAY': None,  # number of minimum number of epochs before the next LR decay (set to None to disable)
                            'LR_GAMMA': 0.5,  # multiplier used for decreasing the LR
                            'SHUFFLE': True,
                            'MAX_FRAME': 20,
                            'EMBEDDING_SIZE': 50,
                            'FUSION_INPUT_LEN': 200,
                            'DROPOUT': 0.5, 
                            'WEIGHT_DECAY': 1e-3,  # L2 regularization
                            'RECURRENT_WEIGHT_DECAY': 0.0,  # L2 regularization in recurrent layers
                            'USE_RECURRENT_DROPOUT': False,  # Use dropout in recurrent layers # DANGEROUS!
                            'RECURRENT_DROPOUT_P': 0.5,  # Percentage of units to drop in recurrent layers
                            'INIT_LAYERS': ['tanh'],
                            'DECODER_HIDDEN_SIZE': 484,  # For models with LSTM decoder (ABiViRNet 484)
                            'TARGET_TEXT_EMBEDDING_SIZE': 99,
                            'ADDITIONAL_OUTPUT_MERGE_MODE': 'sum',
                            'CLASSIFIER_ACTIVATION': 'softmax',
                            'EPSILON': 1e-7,
                            'VOCABULARY_SIZE': 104,
                            'MIN_OCCURRENCES_VOCAB': 0,

                            # Prediction params
                            'SAMPLING_SAVE_MODE': 'numpy',
                            'SAMPLING_RELOAD_POINT': 30,
                            'SAMPLING_RELOAD_EPOCH': True,
                            'TOKENIZATION_METHOD': None,
                            'SAMPLING': 'max_likelihood',  # Possible values: multinomial or max_likelihood (recommended)
                            'TEMPERATURE': 1,
                        },

                        ###################
                        # Model fusion: feature vectors + correlations + menu
                        ###################
                        'FV_C_M_model': {
                            # Model params
                            'MODEL_TYPE': 'FV_C_M_model',  # see predefined models in models.py
                            'MODEL_NAME': 'FV+C+M MODEL', # 'TFG_VIDEO'
                            'STORE_PATH': '%s/FV_C_M_model',  # insert MODELS_ROOT_PATH in %s. '%s/tfg_video'
                            'MODELS_ROOT_PATH': '/models',
                            'EVALUATE': True,
                            # Dataset params
                            'DATA_ROOT_PATH': '/data_yolo',  # '/data' for single images and '/data_video' for group images
                            'DATASET_NAME': 'Food',
                            'REBUILD_DATASET': True,
                            'ALL_CLASSES_NAMES_PATH': '%s/annotations/yolo_classes_new.txt',
                            'YOLO_NAMES': '%s/annotations/yolo.names',
                            'TRAIN_GT_LABELS': '%s/annotations/train_gt_labels.txt',
                            'TRAIN_GT_UNIQUE': '%s/annotations/train_gt_unique.txt',
                            'TRAIN_IMGS_LIST': '%s/annotations/train_imgs.txt',
                            'TEST_GT_LABELS': '%s/annotations/test_gt_labels.txt',
                            'TEST_GT_UNIQUE': '%s/annotations/test_gt_unique.txt',
                            'TEST_IMGS_LIST': '%s/annotations/test_imgs.txt',
                            'INPUTS_IDS_DATASET': ['menu', 'features', 'correlations'],
                            'OUTPUTS_IDS_DATASET': ['output'],
                            'INPUTS': {
                                'menu': {
                                    'type': 'text', # 'raw-image' for single images and 'video' for group images
                                    'words_as_classes': True,  # specific type text param
                                    'max_text_len': 35, # specific type text param
                                    'path': 'annotations/%s_menu.txt',
                                    'num_classes': 104,
                                },
                                'features': {
                                    'type': 'video-features', # 'raw-image' for single images and 'video' for group images
                                    'path': 'annotations/%s_features.txt',
                                    'frames_path': 'annotations/%s_frames.txt', # only for type video,
                                    'indices_path': 'annotations/%s_video_indices.txt', # only for type video,
                                    'max_video_len': 20,
                                    'feat_len': 1536,
                                    'activation': 'tanh',
                                    'normalization': ['L2'],  # For video-features only accepted L2 or None
                                    'data_augmentation': ['noise'],  # For video-features only accepted noise, random_selections or None
                                },
                                'correlations': {
                                    'type': 'text', # 'raw-image' for single images and 'video' for group images
                                    'words_as_classes': True,  # specific type text param
                                    'max_text_len': 20, # specific type text param
                                    'path': 'annotations/%s_pred_labels.txt',
                                    'num_classes': 88,
                                },
                            },
                            'OUTPUTS': {
                                'output': {
                                    'type': 'text',   # 'categorical' for single images and 'text' for group images
                                    'words_as_classes': True,  # specific type text param
                                    'max_text_len': 20, # specific type text param
                                    'path': 'annotations/%s_gt_labels.txt',
                                    'classes': 'annotations/yolo_classes_new.txt',
                                    'metrics': ['multiclass_metrics_custom'],  # multilabel_metrics or multiclass_metrics
                                    'write_type': 'listoflists',  # listoflists or list
                                    'activation': 'softmax',  # sigmoid or softmax
                                    'loss': 'categorical_crossentropy',  # binary_crossentropy or categorical_crossentropy
                                    'save_classes_report': False,  # whether or not to save the classification report for each class
                                }
                            },
                            # Training params
                            'SPLITS': ['test', 'train'], # ['train','val','test']
                            'EMPTY_LABEL': False,
                            'RELOAD': 0,  #30, #39
                            'PATIENCE': 5,
                            'STOP_METRIC': 'accuracy',
                            'BATCH_SIZE': 24,
                            'LR_DECAY': None,  # number of minimum number of epochs before the next LR decay (set to None to disable)
                            'LR_GAMMA': 0.5,  # multiplier used for decreasing the LR
                            'SHUFFLE': True,
                            'MAX_FRAME': 20,
                            'EMBEDDING_SIZE': 50,
                            'FUSION_INPUT_LEN': 200,
                            'DROPOUT': 0.5, 
                            'WEIGHT_DECAY': 1e-3,  # L2 regularization
                            'RECURRENT_WEIGHT_DECAY': 0.0,  # L2 regularization in recurrent layers
                            'USE_RECURRENT_DROPOUT': False,  # Use dropout in recurrent layers # DANGEROUS!
                            'RECURRENT_DROPOUT_P': 0.5,  # Percentage of units to drop in recurrent layers
                            'INIT_LAYERS': ['tanh'],
                            'DECODER_HIDDEN_SIZE': 484,  # For models with LSTM decoder (ABiViRNet 484)
                            'TARGET_TEXT_EMBEDDING_SIZE': 99,
                            'ADDITIONAL_OUTPUT_MERGE_MODE': 'sum',
                            'CLASSIFIER_ACTIVATION': 'softmax',
                            'EPSILON': 1e-4,
                            'VOCABULARY_SIZE': 104,
                            'MIN_OCCURRENCES_VOCAB': 0,

                            # Prediction params
                            'SAMPLING_SAVE_MODE': 'numpy',
                            'SAMPLING_RELOAD_POINT': 30,
                            'SAMPLING_RELOAD_EPOCH': True,
                            'TOKENIZATION_METHOD': None,
                            'SAMPLING': 'max_likelihood',  # Possible values: multinomial or max_likelihood (recommended)
                            'TEMPERATURE': 1,
                        },

                        ###################
                        # Model fusion: feature vectors + yolo predictions
                        ###################
                        'FV_YP_C_model': {
                            # Model params
                            'MODEL_TYPE': 'FV_YP_C_model',  # see predefined models in models.py
                            'MODEL_NAME': 'FV+YP+C MODEL', # 'TFG_VIDEO'
                            'STORE_PATH': '%s/FV_YP_C_model',  # insert MODELS_ROOT_PATH in %s. '%s/tfg_video'
                            'MODELS_ROOT_PATH': '/models',
                            'EVALUATE': True,
                            # Dataset params
                            'DATA_ROOT_PATH': '/data_yolo',  # '/data' for single images and '/data_video' for group images
                            'DATASET_NAME': 'Food',
                            'REBUILD_DATASET': True,
                            'ALL_CLASSES_NAMES_PATH': '%s/annotations/yolo_classes_new.txt',
                            'YOLO_NAMES': '%s/annotations/yolo.names',
                            'TRAIN_GT_LABELS': '%s/annotations/train_gt_labels.txt',
                            'TRAIN_GT_UNIQUE': '%s/annotations/train_gt_unique.txt',
                            'TRAIN_IMGS_LIST': '%s/annotations/train_imgs.txt',
                            'TEST_GT_LABELS': '%s/annotations/test_gt_labels.txt',
                            'TEST_GT_UNIQUE': '%s/annotations/test_gt_unique.txt',
                            'TEST_IMGS_LIST': '%s/annotations/test_imgs.txt',
                            'INPUTS_IDS_DATASET': ['features', 'yolo_predictions', 'correlations'],
                            'OUTPUTS_IDS_DATASET': ['output'],
                            'INPUTS': {
                                'features': {
                                    'type': 'video-features', # 'raw-image' for single images and 'video' for group images
                                    'path': 'annotations/%s_features.txt',
                                    'frames_path': 'annotations/%s_frames.txt', # only for type video,
                                    'indices_path': 'annotations/%s_video_indices.txt', # only for type video,
                                    'max_video_len': 20,
                                    'feat_len': 1536,
                                    'activation': 'tanh',
                                    'normalization': ['L2'],  # For video-features only accepted L2 or None
                                    'data_augmentation': ['noise'],  # For video-features only accepted noise, random_selections or None
                                },
                                'yolo_predictions': {
                                    'type': 'text', # 'raw-image' for single images and 'video' for group images
                                    'words_as_classes': True,  # specific type text param
                                    'max_text_len': 20, # specific type text param
                                    'path': 'annotations/%s_pred_labels.txt',
                                },
                                'correlations': {
                                    'type': 'text', # 'raw-image' for single images and 'video' for group images
                                    'words_as_classes': True,  # specific type text param
                                    'max_text_len': 20, # specific type text param
                                    'path': 'annotations/%s_pred_labels.txt',
                                    'num_classes': 88,
                                },
                            },
                            'OUTPUTS': {
                                'output': {
                                    'type': 'text',   # 'categorical' for single images and 'text' for group images
                                    'words_as_classes': True,  # specific type text param
                                    'max_text_len': 20, # specific type text param
                                    'path': 'annotations/%s_gt_labels.txt',
                                    'classes': 'annotations/yolo_classes_new.txt',
                                    'metrics': ['multiclass_metrics_custom'],  # multilabel_metrics or multiclass_metrics
                                    'write_type': 'listoflists',  # listoflists or list
                                    'activation': 'softmax',  # sigmoid or softmax
                                    'loss': 'categorical_crossentropy',  # binary_crossentropy or categorical_crossentropy
                                    'save_classes_report': False,  # whether or not to save the classification report for each class
                                }
                            },
                            # Training params
                            'SPLITS': ['test', 'train'], # ['train','val','test']
                            'EMPTY_LABEL': False,
                            'RELOAD': 0,  #30, #39
                            'PATIENCE': 5,
                            'STOP_METRIC': 'accuracy',
                            'BATCH_SIZE': 24,
                            'LR_DECAY': None,  # number of minimum number of epochs before the next LR decay (set to None to disable)
                            'LR_GAMMA': 0.5,  # multiplier used for decreasing the LR
                            'SHUFFLE': True,
                            'MAX_FRAME': 20,
                            'EMBEDDING_SIZE': 50,
                            'FUSION_INPUT_LEN': 200,
                            'DROPOUT': 0.5, 
                            'WEIGHT_DECAY': 1e-3,  # L2 regularization
                            'RECURRENT_WEIGHT_DECAY': 0.0,  # L2 regularization in recurrent layers
                            'USE_RECURRENT_DROPOUT': False,  # Use dropout in recurrent layers # DANGEROUS!
                            'RECURRENT_DROPOUT_P': 0.5,  # Percentage of units to drop in recurrent layers
                            'INIT_LAYERS': ['tanh'],
                            'DECODER_HIDDEN_SIZE': 484,  # For models with LSTM decoder (ABiViRNet 484)
                            'TARGET_TEXT_EMBEDDING_SIZE': 99,
                            'ADDITIONAL_OUTPUT_MERGE_MODE': 'sum',
                            'CLASSIFIER_ACTIVATION': 'softmax',
                            'EPSILON': 1e-7,
                            'VOCABULARY_SIZE': 99,
                            'MIN_OCCURRENCES_VOCAB': 0,

                            # Prediction params
                            'SAMPLING_SAVE_MODE': 'numpy',
                            'SAMPLING_RELOAD_POINT': 30,
                            'SAMPLING_RELOAD_EPOCH': True,
                            'TOKENIZATION_METHOD': None,
                            'SAMPLING': 'max_likelihood',  # Possible values: multinomial or max_likelihood (recommended)
                            'TEMPERATURE': 1,
                        },

                        ###################
                        # Model fusion: FV + M
                        ###################
                        'FV_M_model': {
                            # Model params
                            'MODEL_TYPE': 'FV_M_model',  # see predefined models in models.py
                            'MODEL_NAME': 'FV + M MODEL', # 'TFG_VIDEO'
                            'STORE_PATH': '%s/FV_M_model',  # insert MODELS_ROOT_PATH in %s. '%s/tfg_video'
                            'MODELS_ROOT_PATH': '/models',
                            'EVALUATE': True,
                            # Dataset params
                            'DATA_ROOT_PATH': '/data_yolo',  # '/data' for single images and '/data_video' for group images
                            'DATASET_NAME': 'Food',
                            'REBUILD_DATASET': True,
                            'ALL_CLASSES_NAMES_PATH': '%s/annotations/yolo_classes_new.txt',
                            'YOLO_NAMES': '%s/annotations/yolo.names',
                            'TRAIN_GT_LABELS': '%s/annotations/train_gt_labels.txt',
                            'TRAIN_GT_UNIQUE': '%s/annotations/train_gt_unique.txt',
                            'TRAIN_IMGS_LIST': '%s/annotations/train_imgs.txt',
                            'TEST_GT_LABELS': '%s/annotations/test_gt_labels.txt',
                            'TEST_GT_UNIQUE': '%s/annotations/test_gt_unique.txt',
                            'TEST_IMGS_LIST': '%s/annotations/test_imgs.txt',
                            'INPUTS_IDS_DATASET': ['features', 'menu'],
                            'OUTPUTS_IDS_DATASET': ['output'],
                            'INPUTS': {
                                'menu': {
                                    'type': 'text', # 'raw-image' for single images and 'video' for group images
                                    'words_as_classes': True,  # specific type text param
                                    'max_text_len': 35, # specific type text param
                                    'path': 'annotations/%s_menu.txt',
                                    'num_classes': 104,
                                },
                                'features': {
                                    'type': 'video-features', # 'raw-image' for single images and 'video' for group images
                                    'path': 'annotations/%s_features.txt',
                                    'frames_path': 'annotations/%s_frames.txt', # only for type video,
                                    'indices_path': 'annotations/%s_video_indices.txt', # only for type video,
                                    'max_video_len': 20,
                                    'feat_len': 1536,
                                    'activation': 'tanh',
                                    'normalization': ['L2'],  # For video-features only accepted L2 or None
                                    'data_augmentation': ['noise'],  # For video-features only accepted noise, random_selections or None
                                },
                            },
                            'OUTPUTS': {
                                'output': {
                                    'type': 'text',   # 'categorical' for single images and 'text' for group images
                                    'words_as_classes': True,  # specific type text param
                                    'max_text_len': 20, # specific type text param
                                    'path': 'annotations/%s_gt_labels.txt',
                                    'classes': 'annotations/yolo_classes_new.txt',
                                    'metrics': ['multiclass_metrics_custom'],  # multilabel_metrics or multiclass_metrics
                                    'write_type': 'listoflists',  # listoflists or list
                                    'activation': 'softmax',  # sigmoid or softmax
                                    'loss': 'categorical_crossentropy',  # binary_crossentropy or categorical_crossentropy
                                    'save_classes_report': False,  # whether or not to save the classification report for each class
                                }
                            },
                            # Training params
                            'SPLITS': ['test', 'train'], # ['train','val','test']
                            'EMPTY_LABEL': False,
                            'RELOAD': 0,  #30, #39
                            'PATIENCE': 5,
                            'STOP_METRIC': 'accuracy',
                            'BATCH_SIZE': 24,
                            'LR_DECAY': None,  # number of minimum number of epochs before the next LR decay (set to None to disable)
                            'LR_GAMMA': 0.5,  # multiplier used for decreasing the LR
                            'SHUFFLE': True,
                            'MAX_FRAME': 20,
                            'EMBEDDING_SIZE': 50,
                            'FUSION_INPUT_LEN': 200,
                            'DROPOUT': 0.5, 
                            'WEIGHT_DECAY': 1e-3,  # L2 regularization
                            'RECURRENT_WEIGHT_DECAY': 0.0,  # L2 regularization in recurrent layers
                            'USE_RECURRENT_DROPOUT': False,  # Use dropout in recurrent layers # DANGEROUS!
                            'RECURRENT_DROPOUT_P': 0.5,  # Percentage of units to drop in recurrent layers
                            'INIT_LAYERS': ['tanh'],
                            'DECODER_HIDDEN_SIZE': 484,  # For models with LSTM decoder (ABiViRNet 484)
                            'TARGET_TEXT_EMBEDDING_SIZE': 99,
                            'ADDITIONAL_OUTPUT_MERGE_MODE': 'sum',
                            'CLASSIFIER_ACTIVATION': 'softmax',
                            'EPSILON': 1e-7,
                            'VOCABULARY_SIZE': 104,
                            'MIN_OCCURRENCES_VOCAB': 0,

                            # Prediction params
                            'SAMPLING_SAVE_MODE': 'numpy',
                            'SAMPLING_RELOAD_POINT': 30,
                            'SAMPLING_RELOAD_EPOCH': True,
                            'TOKENIZATION_METHOD': None,
                            'SAMPLING': 'max_likelihood',  # Possible values: multinomial or max_likelihood (recommended)
                            'TEMPERATURE': 1,
                        },

                        ###################
                        # Model fusion: feature vectors + yolo predictions
                        ###################
                        'FV_YP_model': {
                            # Model params
                            'MODEL_TYPE': 'FV_YP_model',  # see predefined models in models.py
                            'MODEL_NAME': 'FV+YP MODEL', # 'TFG_VIDEO'
                            'STORE_PATH': '%s/FV_YP_model',  # insert MODELS_ROOT_PATH in %s. '%s/tfg_video'
                            'MODELS_ROOT_PATH': '/models',
                            'EVALUATE': True,
                            # Dataset params
                            'DATA_ROOT_PATH': '/data_yolo',  # '/data' for single images and '/data_video' for group images
                            'DATASET_NAME': 'Food',
                            'REBUILD_DATASET': True,
                            'ALL_CLASSES_NAMES_PATH': '%s/annotations/yolo_classes_new.txt',
                            'YOLO_NAMES': '%s/annotations/yolo.names',
                            'TRAIN_GT_LABELS': '%s/annotations/train_gt_labels.txt',
                            'TRAIN_GT_UNIQUE': '%s/annotations/train_gt_unique.txt',
                            'TRAIN_IMGS_LIST': '%s/annotations/train_imgs.txt',
                            'TEST_GT_LABELS': '%s/annotations/test_gt_labels.txt',
                            'TEST_GT_UNIQUE': '%s/annotations/test_gt_unique.txt',
                            'TEST_IMGS_LIST': '%s/annotations/test_imgs.txt',
                            'INPUTS_IDS_DATASET': ['features', 'yolo_predictions'],
                            'OUTPUTS_IDS_DATASET': ['output'],
                            'INPUTS': {
                                'features': {
                                    'type': 'video-features', # 'raw-image' for single images and 'video' for group images
                                    'path': 'annotations/%s_features.txt',
                                    'frames_path': 'annotations/%s_frames.txt', # only for type video,
                                    'indices_path': 'annotations/%s_video_indices.txt', # only for type video,
                                    'max_video_len': 20,
                                    'feat_len': 1536,
                                    'activation': 'tanh',
                                    'normalization': ['L2'],  # For video-features only accepted L2 or None
                                    'data_augmentation': ['noise'],  # For video-features only accepted noise, random_selections or None
                                },
                                'yolo_predictions': {
                                    'type': 'text', # 'raw-image' for single images and 'video' for group images
                                    'words_as_classes': True,  # specific type text param
                                    'max_text_len': 20, # specific type text param
                                    'path': 'annotations/%s_pred_labels.txt',
                                }
                            },
                            'OUTPUTS': {
                                'output': {
                                    'type': 'text',   # 'categorical' for single images and 'text' for group images
                                    'words_as_classes': True,  # specific type text param
                                    'max_text_len': 20, # specific type text param
                                    'path': 'annotations/%s_gt_labels.txt',
                                    'classes': 'annotations/yolo_classes_new.txt',
                                    'metrics': ['multiclass_metrics_custom'],  # multilabel_metrics or multiclass_metrics
                                    'write_type': 'listoflists',  # listoflists or list
                                    'activation': 'softmax',  # sigmoid or softmax
                                    'loss': 'categorical_crossentropy',  # binary_crossentropy or categorical_crossentropy
                                    'save_classes_report': False,  # whether or not to save the classification report for each class
                                }
                            },
                            # Training params
                            'SPLITS': ['test', 'train'], # ['train','val','test']
                            'EMPTY_LABEL': False,
                            'RELOAD': 0,  #30, #39
                            'PATIENCE': 5,
                            'STOP_METRIC': 'accuracy',
                            'BATCH_SIZE': 24,
                            'LR_DECAY': None,  # number of minimum number of epochs before the next LR decay (set to None to disable)
                            'LR_GAMMA': 0.5,  # multiplier used for decreasing the LR
                            'SHUFFLE': True,
                            'MAX_FRAME': 20,
                            'EMBEDDING_SIZE': 50,
                            'FUSION_INPUT_LEN': 200,
                            'DROPOUT': 0.5, 
                            'WEIGHT_DECAY': 1e-3,  # L2 regularization
                            'RECURRENT_WEIGHT_DECAY': 0.0,  # L2 regularization in recurrent layers
                            'USE_RECURRENT_DROPOUT': False,  # Use dropout in recurrent layers # DANGEROUS!
                            'RECURRENT_DROPOUT_P': 0.5,  # Percentage of units to drop in recurrent layers
                            'INIT_LAYERS': ['tanh'],
                            'DECODER_HIDDEN_SIZE': 484,  # For models with LSTM decoder (ABiViRNet 484)
                            'TARGET_TEXT_EMBEDDING_SIZE': 99,
                            'ADDITIONAL_OUTPUT_MERGE_MODE': 'sum',
                            'CLASSIFIER_ACTIVATION': 'softmax',
                            'EPSILON': 1e-4,
                            'VOCABULARY_SIZE': 99,
                            'MIN_OCCURRENCES_VOCAB': 0,

                            # Prediction params
                            'SAMPLING_SAVE_MODE': 'numpy',
                            'SAMPLING_RELOAD_POINT': 30,
                            'SAMPLING_RELOAD_EPOCH': True,
                            'TOKENIZATION_METHOD': None,
                            'SAMPLING': 'max_likelihood',  # Possible values: multinomial or max_likelihood (recommended)
                            'TEMPERATURE': 1,
                        },

                        ###################
                        # Model fusion: FV + C
                        ###################
                        'FV_C_model': {
                            # Model params
                            'MODEL_TYPE': 'FV_C_model',  # see predefined models in models.py
                            'MODEL_NAME': 'FV + C MODEL', # 'TFG_VIDEO'
                            'STORE_PATH': '%s/FV_C_model',  # insert MODELS_ROOT_PATH in %s. '%s/tfg_video'
                            'MODELS_ROOT_PATH': '/models',
                            'EVALUATE': True,
                            # Dataset params
                            'DATA_ROOT_PATH': '/data_yolo',  # '/data' for single images and '/data_video' for group images
                            'DATASET_NAME': 'Food',
                            'REBUILD_DATASET': True,
                            'ALL_CLASSES_NAMES_PATH': '%s/annotations/yolo_classes_new.txt',
                            'YOLO_NAMES': '%s/annotations/yolo.names',
                            'TRAIN_GT_LABELS': '%s/annotations/train_gt_labels.txt',
                            'TRAIN_GT_UNIQUE': '%s/annotations/train_gt_unique.txt',
                            'TRAIN_IMGS_LIST': '%s/annotations/train_imgs.txt',
                            'TEST_GT_LABELS': '%s/annotations/test_gt_labels.txt',
                            'TEST_GT_UNIQUE': '%s/annotations/test_gt_unique.txt',
                            'TEST_IMGS_LIST': '%s/annotations/test_imgs.txt',
                            'INPUTS_IDS_DATASET': ['features', 'correlations'],
                            'OUTPUTS_IDS_DATASET': ['output'],
                            'INPUTS': {
                                'features': {
                                    'type': 'video-features', # 'raw-image' for single images and 'video' for group images
                                    'path': 'annotations/%s_features.txt',
                                    'frames_path': 'annotations/%s_frames.txt', # only for type video,
                                    'indices_path': 'annotations/%s_video_indices.txt', # only for type video,
                                    'max_video_len': 20,
                                    'feat_len': 1536,
                                    'activation': 'tanh',
                                    'normalization': ['L2'],  # For video-features only accepted L2 or None
                                    'data_augmentation': ['noise'],  # For video-features only accepted noise, random_selections or None
                                },
                                'correlations': {
                                    'type': 'text', # 'raw-image' for single images and 'video' for group images
                                    'words_as_classes': True,  # specific type text param
                                    'max_text_len': 20, # specific type text param
                                    'path': 'annotations/%s_pred_labels.txt',
                                    'num_classes': 88,
                                },
                            },
                            'OUTPUTS': {
                                'output': {
                                    'type': 'text',   # 'categorical' for single images and 'text' for group images
                                    'words_as_classes': True,  # specific type text param
                                    'max_text_len': 20, # specific type text param
                                    'path': 'annotations/%s_gt_labels.txt',
                                    'classes': 'annotations/yolo_classes_new.txt',
                                    'metrics': ['multiclass_metrics_custom'],  # multilabel_metrics or multiclass_metrics
                                    'write_type': 'listoflists',  # listoflists or list
                                    'activation': 'softmax',  # sigmoid or softmax
                                    'loss': 'categorical_crossentropy',  # binary_crossentropy or categorical_crossentropy
                                    'save_classes_report': False,  # whether or not to save the classification report for each class
                                }
                            },
                            # Training params
                            'SPLITS': ['test', 'train'], # ['train','val','test']
                            'EMPTY_LABEL': False,
                            'RELOAD': 0,  #30, #39
                            'PATIENCE': 5,
                            'STOP_METRIC': 'accuracy',
                            'BATCH_SIZE': 24,
                            'LR_DECAY': None,  # number of minimum number of epochs before the next LR decay (set to None to disable)
                            'LR_GAMMA': 0.5,  # multiplier used for decreasing the LR
                            'SHUFFLE': True,
                            'MAX_FRAME': 20,
                            'EMBEDDING_SIZE': 44,
                            'FUSION_INPUT_LEN': 200,
                            'DROPOUT': 0.5, 
                            'WEIGHT_DECAY': 1e-3,  # L2 regularization
                            'RECURRENT_WEIGHT_DECAY': 0.0,  # L2 regularization in recurrent layers
                            'USE_RECURRENT_DROPOUT': False,  # Use dropout in recurrent layers # DANGEROUS!
                            'RECURRENT_DROPOUT_P': 0.5,  # Percentage of units to drop in recurrent layers
                            'INIT_LAYERS': ['tanh'],
                            'DECODER_HIDDEN_SIZE': 484,  # For models with LSTM decoder (ABiViRNet 484)
                            'TARGET_TEXT_EMBEDDING_SIZE': 99,
                            'ADDITIONAL_OUTPUT_MERGE_MODE': 'sum',
                            'CLASSIFIER_ACTIVATION': 'softmax',
                            'EPSILON': 1e-4,
                            'VOCABULARY_SIZE': 99,
                            'MIN_OCCURRENCES_VOCAB': 0,

                            # Prediction params
                            'SAMPLING_SAVE_MODE': 'numpy',
                            'SAMPLING_RELOAD_POINT': 30,
                            'SAMPLING_RELOAD_EPOCH': True,
                            'TOKENIZATION_METHOD': None,
                            'SAMPLING': 'max_likelihood',  # Possible values: multinomial or max_likelihood (recommended)
                            'TEMPERATURE': 1,
                        },

                        ###################
                        # Model with only features as input
                        ###################
                        'FV_model': {
                            # Model params
                            'MODEL_TYPE': 'FV_model',  # see predefined models in models.py
                            'MODEL_NAME': 'FV model',
                            'STORE_PATH': '%s/test',  # insert MODELS_ROOT_PATH in %s. '%s/tfg_video'
                            'MODELS_ROOT_PATH': '/models',
                            'EVALUATE': True,
                            # Dataset params
                            'DATA_ROOT_PATH': '/data_yolo',  # '/data' for single images and '/data_video' for group images
                            'DATASET_NAME': 'Food',
                            'REBUILD_DATASET': True,
                            'ALL_CLASSES_NAMES_PATH': '%s/annotations/yolo_classes_new.txt',
                            'YOLO_NAMES': '%s/annotations/yolo.names',
                            'TRAIN_GT_LABELS': '%s/annotations/train_gt_labels.txt',
                            'TRAIN_GT_UNIQUE': '%s/annotations/train_gt_unique.txt',
                            'TRAIN_IMGS_LIST': '%s/annotations/train_imgs.txt',
                            'TEST_GT_LABELS': '%s/annotations/test_gt_labels.txt',
                            'TEST_GT_UNIQUE': '%s/annotations/test_gt_unique.txt',
                            'TEST_IMGS_LIST': '%s/annotations/test_imgs.txt',
                            'INPUTS_IDS_DATASET': ['features'],
                            'OUTPUTS_IDS_DATASET': ['output'],
                            'INPUTS': {
                                'features': {
                                    'type': 'video-features', # 'raw-image' for single images and 'video' for group images
                                    'path': 'annotations/%s_features.txt',
                                    'frames_path': 'annotations/%s_frames.txt', # only for type video,
                                    'indices_path': 'annotations/%s_video_indices.txt', # only for type video,
                                    'max_video_len': 20,
                                    'feat_len': 1536,
                                    'activation': 'tanh',
                                    'normalization': ['L2'],  # For video-features only accepted L2 or None
                                    'data_augmentation': ['noise'],  # For video-features only accepted noise, random_selections or None
                                }
                            },
                            'OUTPUTS': {
                                'output': {
                                    'type': 'text',   # 'categorical' for single images and 'text' for group images
                                    'words_as_classes': True,  # specific type text param
                                    'max_text_len': 20, # specific type text param
                                    'path': 'annotations/%s_gt_labels.txt',
                                    'classes': 'annotations/yolo_classes_new.txt',
                                    'metrics': ['multiclass_metrics_custom'],  # multilabel_metrics or multiclass_metrics
                                    'write_type': 'listoflists',  # listoflists or list
                                    'activation': 'softmax',  # sigmoid or softmax
                                    'loss': 'categorical_crossentropy',  # binary_crossentropy or categorical_crossentropy
                                    'save_classes_report': False,  # whether or not to save the classification report for each class
                                }
                            },
                            # Training params
                            'SPLITS': ['test', 'train'], # ['train','val','test']
                            'EMPTY_LABEL': False,
                            'RELOAD': 0,  #30, #39
                            'PATIENCE': 5,
                            'STOP_METRIC': 'accuracy',
                            'BATCH_SIZE': 24,
                            'LR_DECAY': None,  # number of minimum number of epochs before the next LR decay (set to None to disable)
                            'LR_GAMMA': 0.5,  # multiplier used for decreasing the LR
                            'SHUFFLE': True,
                            'MAX_FRAME': 20,
                            'EMBEDDING_SIZE': 99,  #size of output_real vocabulary
                            'FUSION_INPUT_LEN': 200,
                            'DROPOUT': 0.5, 
                            'WEIGHT_DECAY': 1e-3,  # L2 regularization
                            'RECURRENT_WEIGHT_DECAY': 0.0,  # L2 regularization in recurrent layers
                            'USE_RECURRENT_DROPOUT': False,  # Use dropout in recurrent layers # DANGEROUS!
                            'RECURRENT_DROPOUT_P': 0.5,  # Percentage of units to drop in recurrent layers
                            'INIT_LAYERS': ['tanh'],
                            'DECODER_HIDDEN_SIZE': 484,  # For models with LSTM decoder (ABiViRNet 484)
                            'TARGET_TEXT_EMBEDDING_SIZE': 99,
                            'ADDITIONAL_OUTPUT_MERGE_MODE': 'sum',
                            'CLASSIFIER_ACTIVATION': 'softmax',
                            'EPSILON': 1e-4,
                            'VOCABULARY_SIZE': 99,
                            'MIN_OCCURRENCES_VOCAB': 0,

                            # Prediction params
                            'SAMPLING_SAVE_MODE': 'numpy',
                            'SAMPLING_RELOAD_POINT': 30,
                            'SAMPLING_RELOAD_EPOCH': True,
                            'TOKENIZATION_METHOD': None,
                            'SAMPLING': 'max_likelihood',  # Possible values: multinomial or max_likelihood (recommended)
                            'TEMPERATURE': 1,
                        },
    }

    # ============================================
    parameters = locals().copy()
    return parameters
