import cv2 
import os, copy, shutil
import numpy as np
from os import listdir
from os.path import isfile, join


################################
##         PARAMETERS         ##
################################

min_IoU = 0.5
max_pred_bboxes = 20 

# Train split folds
train_folds = [5, 18, 22]
for i in range(25, 98):	
	if i != 28 and i != 74:	 # folds 28 and 74 contain problems
		train_folds.append(i)

# Test split folds
test_folds = []
for i in range(1, 25):
	if i!=5 and i!=18 and i!=22:
		test_folds.append(i)

splits = ['train','test']

origin_path = "tray_images_dataset/"
dst_path = "data_yolo/"
images_folder_out = 'images/'
features_folder_out = 'features/'
images_suffix = '%s_imgs.txt'
annotations_folder = 'annotations/'
annotations_suffix = '%s_labels.txt'
all_classes_names_path = "yolo_classes_new.txt"
yolo_names = "yolo.names"
test_menu_ann = "data_menu/annotations/test_menu.txt"
train_menu_ann = "data_menu/annotations/train_menu.txt"


def create_folders(path, images_folder, annotations_folder, all_classes_names_path):

	im_path = path + images_folder
	ann_path = path + annotations_folder

	# Clean directory if exists
	if os.path.isdir(path):
		try:
			shutil.rmtree(path)

			# Create destination paths
			os.mkdir(path)

		except:
			print "Couldn't clean all the directory " + path + ". Skipping it..."

			print "Cleaning images and annotations folders..."
			if os.path.isdir(im_path):
				shutil.rmtree(im_path)
			if os.path.isdir(ann_path):
				shutil.rmtree(ann_path)


	os.mkdir(im_path)
	for split in splits:
		os.mkdir(im_path + '/' + split)

	os.mkdir(ann_path)

	# Read all classes names and create classes dictonary    
	with open(all_classes_names_path, 'r') as cn:
		classes_list = cn.readlines()

	dic_classes = {}
	for i in range(len(classes_list)):
		dic_classes[classes_list[i].strip('\n')] = i

	print dic_classes
	print "num of classes: " + str(len(dic_classes))
	print "max index: " + str(max(dic_classes.values()))

	return im_path, ann_path, dic_classes


def check_overlap_and_get_max_conf(check_bbox, list_bboxes, min_IoU, list_probs):
    """
    Returns index of list_bboxes with the maximum overlap with check_bbox. Minimum overlap required is min_IoU.
    Return index -1 if there isn't solution.
    """
    list_copy = [list(b) for b in list_bboxes]
    list_probs_copy = list(list_probs)
    ind = -1
    maxV = 0
    for i, b in enumerate(list_copy):
    	iou = compute_iou(b, check_bbox)
        if iou > min_IoU and list_probs_copy[i] > maxV:
            ind = i
            maxV = list_probs_copy[i]

    return ind


def check_overlap_and_get_max_iou(check_bbox, list_bboxes, min_IoU):
    """
    Returns index of list_bboxes with the maximum overlap with check_bbox. Minimum overlap required is min_IoU.
    Return index -1 if there isn't solution.
    """
    # Lists copies
    list_copy = [list(b) for b in list_bboxes]
    check_bbox_copy = list(check_bbox)
    #iou_list = []

    ind = -1
    maxV = min_IoU

    for i, b in enumerate(list_copy):
    	check_bbox_copy = list(check_bbox)
    	iou = compute_iou(b, check_bbox_copy)
    	#iou_list.append(iou)
        if iou > maxV:
            ind = i
            maxV = iou

    #iou_list = sorted(iou_list, reverse=True)

    # If there are two predictions with very similiar iou and one of them is > 0.3 (-> precited bbox contains two or more valid gts)
    #if len(iou_list) >= 2 and iou_list[0] - iou_list[1] < 0.1 and iou_list[0]>0.3:
    #	ind = -2

    return ind


def compute_iou(c, a):
    """
    Calculates the intersection over union (IoU) over two bboxes, c and a.

    :param c: control bbox, [x, y, w, h]
    :param a: annotation bbox, [x, y, w, h]
    :return: IoU value
    """
    # Convert w and h to max x, y
    c[3] = c[3] + c[1]
    c[2] = c[2] + c[0]
    a[3] = a[3] + a[1]
    a[2] = a[2] + a[0]

    # intersection
    ixmin = np.maximum(c[0], a[0])
    iymin = np.maximum(c[1], a[1])
    ixmax = np.minimum(c[2], a[2])
    iymax = np.minimum(c[3], a[3])
    iw = np.maximum(ixmax - ixmin + 1., 0.)
    ih = np.maximum(iymax - iymin + 1., 0.)
    inters = iw * ih

    # union
    uni = ((a[2] - a[0] + 1.) * (a[3] - a[1] + 1.) +
           (c[2] - c[0] + 1.) *
           (c[3] - c[1] + 1.) - inters)

    return inters / uni


def format_yolo_bbox(pred_values, imw=None, imh=None):
    """
    Format yolo prediction (relative and central x, y and relative w, h) to normal bbox (x, y top left and absolute w, h)

    :param pred_values: [x, y, w, h] relative and central x, y and relative w, h
    :param imw: image width
    :param imh: image height
    :return: [x, y, w, h] (x, y top left and absolute w, h)
    """

    bbox = [0, 0, 0, 0]
    x, y, w, h = pred_values
    bbox[0] = x - w / 2
    bbox[1] = y - h / 2
    bbox[2] = w
    bbox[3] = h

    if imw is not None and imh is not None:
        bbox[0] = int(bbox[0] * imw)
        bbox[1] = int(bbox[1] * imh)
        bbox[2] = int(bbox[2] * imw)
        bbox[3] = int(bbox[3] * imh)

    return bbox



if __name__ == '__main__':

	# Create folders
	print('Creating folders')
	im_path, ann_path, dic_classes = create_folders(dst_path, images_folder_out, annotations_folder, all_classes_names_path)

	# Add menu files to annotations
	shutil.copyfile(test_menu_ann, ann_path + "test_menu.txt")
	shutil.copyfile(train_menu_ann, ann_path + "train_menu.txt")

	# Add yolo classes to annotations
	shutil.copyfile(all_classes_names_path, ann_path + "yolo_classes_new.txt")
	shutil.copyfile(yolo_names, ann_path + "yolo.names")

	# Add mappings to dic_classes (there are different yolo classes that actually are the same)
	dic_classes['cuscus'] = dic_classes['couscous']
	dic_classes['Canalones'] = dic_classes['canelones']
	dic_classes['canalones'] = dic_classes['canelones']
	dic_classes['ensalada de lentejas '] = dic_classes['ensalada de lentejas']
	dic_classes['fideu\xc3\xa1 '] = dic_classes['fideu\xc3\xa1']
	dic_classes['arroz bolonesa'] = dic_classes['arroz a la bolo\xc3\xb1esa']
	dic_classes['arroz con bolonesa'] = dic_classes['arroz a la bolo\xc3\xb1esa']
	dic_classes['Escalopa de ternera'] = dic_classes['escalopa de ternera']
	dic_classes['escalopa de ternera '] = dic_classes['escalopa de ternera']
	dic_classes['Bistec de ternera a la pimienta'] = dic_classes['bistec de ternera a la pimienta']
	dic_classes['ensalada de cuscus'] = dic_classes['ensalada de couscous']
	dic_classes['espinacas a la crema '] = dic_classes['espinacas a la crema']
	dic_classes['empanadillas '] = dic_classes['empanadillas']
	dic_classes['espaguetis napolitana'] = dic_classes['espaguetis']
	dic_classes['tallarines al pesto'] = dic_classes['espaguetis']

	with open(yolo_names, 'r') as g:
		classes_list = g.readlines()
		yolo_classes_list = [c.strip('\n') for c in classes_list]
		g.close()


	for split in splits:

		gt_f = open(ann_path + split +'_gt_labels.txt', 'w')
		img_f = open(ann_path + split + '_imgs.txt', 'w')
		frames_f = open(ann_path + split + '_frames.txt', 'w')
		ind_f = open(ann_path + split + '_video_indices.txt', 'w')
		pred_f = open(ann_path + split + '_pred_labels.txt', 'w')
		feat_f = open(ann_path + split + '_features.txt', 'w')
		gt_ind_f = open(ann_path + split +'_gt_ind.txt', 'w')
		gt_unique_f = open(ann_path + split +'_gt_unique.txt', 'w')
		yolo_preds_f = open(ann_path + split +'_yolo_preds_probs.txt', 'w')
		
		if split == 'train':
			folds = train_folds
		else:
			folds = test_folds

		indx = 0

		for n in folds:

			folder_name = "new_data_test" + str(n)

			if os.path.isdir(origin_path + folder_name):
				os.mkdir(dst_path + images_folder_out + split + '/' + str(n))
				path = origin_path + folder_name + "/images/test/"
				onlyfiles = [f for f in listdir(path) if isfile(join(path, f))]

				labels_path = origin_path + folder_name + "/labels/test/"
				preds_path = origin_path + "new_data_test" + str(n) + "/prediction/preds_9000_test/"
				
				preds_post_path = origin_path + "new_data_test" + str(n) + "/prediction/preds_post_iou/"
				if not os.path.isdir(preds_post_path):
					os.mkdir(preds_post_path)				

				if os.path.isdir(origin_path + folder_name + "/train_cfg/"):
					if os.path.isfile(origin_path + folder_name + "/train_cfg/new_data_test" + str(n) + ".names"):
						with open(origin_path + folder_name + "/train_cfg/new_data_test" + str(n) + ".names", 'r') as g:
							classes_list = g.readlines()
							classes_list = [c.strip('\n') for c in classes_list]

						for f in onlyfiles:
							f_id = f[:-4]

							# Save image in set directory 
							# Read RGB image 
							img = cv2.imread(path + f)

							#print(img.shape) # (768, 1024, 3)
							img_h = img.shape[0]
							img_w = img.shape[1]

							with open(labels_path + f_id + ".txt", 'r') as f, open(preds_path + f_id + ".txt", 'r') as g:
							    gt_content = f.readlines()
							    preds_content = g.readlines()
							    f.close()
							    g.close()

							preds_post_f = open(preds_post_path + f_id + ".txt", 'w')

							gt_sentence = ""
							gt_labels = []
							gt_bboxes = []
							pred_sentence = ""
							pred_labels = []
							pred_bboxes = []
							pred_probs = []

							if len(gt_content) > 0:
								for l in gt_content:
									l = l.split()
									gt_labels.append(dic_classes[classes_list[int(l[0])]])
									gt_bboxes.append([int(float(l[1])*img_w), int(float(l[2])*img_h), int(float(l[3])*img_w), int(float(l[4])*img_h)])

							if len(preds_content) > 0:
								for l in preds_content:
									l = l.split()
									if float(l[1])<0 or float(l[2])<0 or float(l[3])<0 or float(l[4])<0:
										print "pred with negative values, discarded!"
									else:
										pred_labels.append(dic_classes[yolo_classes_list[int(l[0])]])
										pred_bboxes.append([int(float(l[1])*img_w), int(float(l[2])*img_h), int(float(l[3])*img_w), int(float(l[4])*img_h)])								
										pred_probs.append(float(l[5]))

							num_gts = len(gt_labels)
							num_preds = len(pred_labels)

							# Piece of code to save ground truth bounding boxes (used to check)
							#for i in range(num_gts):

							#	x = gt_bboxes[i][0]
							#	y = gt_bboxes[i][1]
							#	w = gt_bboxes[i][2]
							#	h = gt_bboxes[i][3]

							#	crop = img[y-h/2:y+h/2,x-w/2:x+w/2].copy()
							#	new_image_name = images_folder_out + split + '/' + str(n) + '/' + str(f_id) + '_' + str(i) + '_gt.jpg'
							#	print >> img_f, new_image_name
							#	cv2.imwrite(dst_path + new_image_name, crop)
							
							if num_preds > 20:
								num_preds = 20

							counts = 0
							for i in range(num_preds):
								# We assume that preds are sorted by prob

								# Get index of best GT 
								ind = check_overlap_and_get_max_iou(pred_bboxes[i], gt_bboxes, min_IoU)


								if ind == -1: # There isn't a GT bbox and label that matches the prediction, so we set the GT of this pred to background
									gt_sentence += str(0) + " "
									
									# Save the gt individually (used for training the baseline CNN)
									print >> gt_ind_f, str(0)

								#elif ind == -2:
									# Found a predicted bbox with more than one valid gt, so we discard it
								#	continue

								else:  # Exists a GT bbox and label that matches the prediction
									gt_sentence += str(gt_labels[ind]) + " "

									# Save the gt individually (used for training the baseline CNN)
									print >> gt_ind_f, str(gt_labels[ind])


								# Save pred label and img
								pred_sentence += str(pred_labels[i]) + " "

								x = float(pred_bboxes[i][0])/float(img_w)
								y = float(pred_bboxes[i][1])/float(img_h)
								w = float(pred_bboxes[i][2])/float(img_w)
								h = float(pred_bboxes[i][3])/float(img_h)

								print >> preds_post_f, str(pred_labels[i]) + " " + str(x) + " " + str(y) + " " + str(w) + " " + str(h) + " " + str(pred_probs[i])
								print >> yolo_preds_f, str(pred_labels[i]) + " " + str(pred_probs[i])

								x = pred_bboxes[i][0]
								y = pred_bboxes[i][1]
								w = pred_bboxes[i][2]
								h = pred_bboxes[i][3]
								
								# Save bbox and path to it
								crop = img[y-h/2:y+h/2,x-w/2:x+w/2].copy()
								new_image_name = images_folder_out + split + '/' + str(n) + '/' + str(f_id) + '_' + str(i) + '.jpg'
								print >> img_f, new_image_name
								cv2.imwrite(dst_path + new_image_name, crop)
								
								# Save features vector path of images (not yet generated)
								new_feat_name = features_folder_out + split + '/' + str(n) + '/' + str(f_id) + '_' + str(i) + '.npy'
								print >> feat_f, new_feat_name

								counts += 1

							preds_post_f.close()

							if counts > 0:

								# Save gt sentence, pred sentence, frame and index
								print >> gt_f, gt_sentence
								print >> pred_f, pred_sentence
								print >> frames_f, str(counts)
								print >> ind_f, str(indx)

								indx += counts

								unique_gt = ""
								for gt in gt_labels:
									unique_gt += str(gt) + " "
								print >> gt_unique_f, unique_gt


				print "Fold " + str(n) + " finished!"

			else:
				print origin_path + "new_data_test" + str(n) + ".names doesn't exist!" 


		gt_f.close()
		img_f.close()
		frames_f.close()
		ind_f.close()
		pred_f.close()
		feat_f.close()
		gt_ind_f.close()
		gt_unique_f.close()
		yolo_preds_f.close()

